import select
import socket
import sys
import time

from Homevee.Helper import Logger

XBOX_PORT = 5050
XBOX_PING = "dd00000a000000000000000400000002"

py3 = sys.version_info[0] > 2

class XBOXOneAPI:
    def __init__(self):
        return

    def xbox_wake_up(self, ip_address, live_id):
        Logger.log("Starting Xbox")

        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.setblocking(0)
        s.bind(("", 0))
        s.connect((ip_address, XBOX_PORT))

        if isinstance(live_id, str):
            live_id = live_id.encode()
        else:
            live_id = live_id

        power_payload = b'\x00' + chr(len(live_id)).encode() + live_id + b'\x00'
        power_header = b'\xdd\x02\x00' + chr(len(power_payload)).encode() + b'\x00\x00'
        power_packet = power_header + power_payload
        Logger.log("Sending power on packets to {0}...".format(ip_address))
        self.send_power(s, power_packet)
        # ping_result = send_ping(s)

        s.close()


    def send_power(self, s, data, times=5):
        for i in range(0, times):
            s.send(data)
            time.sleep(1)


    def send_ping(self, s):
        s.send(bytearray.fromhex(XBOX_PING))
        return select.select([s], [], [], 5)[0]



#!/usr/bin/python
# -*- coding: utf-8 -*-

from Homevee.Item import Item
from Homevee.Item.Device import Device
from Homevee.Item.Person import Person
from Homevee.Utils.Database import Database

class PersonManager():
    def __init__(self):
        return

    def add_edit_person(self, user, id, name, nickname, address, latitude, longitude, phonenumber, birthdate, db: Database = None):
        if db is None:
            db = Database()

        person_item = None

        try:
            person_item = Item.load_from_db(Person, id, db)
        except:
            pass

        if person_item is None:
            person_item = Person(name, nickname, phonenumber, address, birthdate, longitude, latitude)
        else:
            person_item.name = name
            person_item.nickname = nickname
            person_item.phone_number = phonenumber
            person_item.address = address
            person_item.birthdate = birthdate
            person_item.longitude = longitude
            person_item.latitude = latitude
            print("update")

        return person_item.save()

    def get_persons(self, db):
        persons = Person.load_all(db)
        return persons

    def delete_person(self, user, id, db: Database = None):
        if db is None:
            db = Database()
        person = Device.load_from_db(Person, id)
        return person.delete()


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.WeatherManager import WeatherManager

ACTION_KEY_GET_WEATHER = "getweather"
ACTION_KEY_GET_WEATHER_CITY_LIST = "getweathercitylist"
ACTION_KEY_SET_WEATHER_CITY_ID = "setweathercityid"

class WeatherAPIModule(APIModule):
    def __init__(self):
        super(WeatherAPIModule, self).__init__()
        self.weather_manager = WeatherManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_WEATHER: self.get_weather,
            ACTION_KEY_GET_WEATHER_CITY_LIST: self.get_weather_city_list,
            ACTION_KEY_SET_WEATHER_CITY_ID: self.set_weather_city_id
        }

        return mappings

    def get_weather(self, user, request, db):
        try:
            data = self.weather_manager.get_weather(int(request['daycount']), db)
            if data is not None:
                return Status(type=STATUS_OK, data={'weather':data})
            else:
                return Status(type=STATUS_ERROR)
        except:
            return Status(type=STATUS_ERROR)

    def get_weather_city_list(self, user, request, db):
        try:
            data = self.weather_manager.get_weather_city_list(db)
            if data is not None:
                return Status(type=STATUS_OK, data=data)
            else:
                return Status(type=STATUS_ERROR)
        except:
            return Status(type=STATUS_ERROR)

    def set_weather_city_id(self, user, request, db):
        try:
            data = self.weather_manager.set_weather_city_id(user, request['id'], db)
            if data is not None:
                return Status(type=STATUS_OK, data=data)
            else:
                return Status(type=STATUS_ERROR)
        except:
            return Status(type=STATUS_ERROR)


from Homevee.APIModule import APIModule
from Homevee.Item.Person import Person
from Homevee.Item.Status import *
from Homevee.Manager.PersonManager import PersonManager

ACTION_KEY_GET_PERSONS = "getpersons"
ACTION_KEY_ADD_EDIT_PERSON = "addeditperson"
ACTION_KEY_DELETE_PERSON = "deleteperson"

class PersonAPIModule(APIModule):
    def __init__(self):
        super(PersonAPIModule, self).__init__()
        self.person_manager = PersonManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_PERSONS: self.get_persons,
            ACTION_KEY_ADD_EDIT_PERSON: self.add_edit_person,
            ACTION_KEY_DELETE_PERSON: self.delete_person
        }

        return mappings

    def get_persons(self, user, request, db):
        persons = self.person_manager.get_persons(db)
        return Status(type=STATUS_OK, data={'persons': Person.list_to_dict(persons)})

    def add_edit_person(self, user, request, db):
        self.person_manager.add_edit_person(user, request['id'], request['name'], request['nickname'],
                                                       request['address'],
                                                       request['latitude'], request['longitude'],
                                                       request['phonenumber'],
                                                       request['birthdate'], db)
        return Status(type=STATUS_OK)

    def delete_person(self, user, request, db):
        self.person_manager.delete_person(user, request['id'], db)
        return Status(type=STATUS_OK)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

from Homevee.Helper import Logger
from Homevee.Item.Gateway import *
from Homevee.Item.Status import *
from Homevee.Manager.RoomManager import RoomManager
from Homevee.Manager.SensorDataManager import SENSOR_TYPE_MAP
from Homevee.Utils.DeviceTypes import *


class DeviceManager:
    def __init__(self):
        return

    '''
function getGatewayDevices($type, $db){
	switch($type){
		case "Network":
			$networkDevices = array();

			exec("arp", $hostnames);
			exec("arp -n", $ipAdresses);

			for($i = 1; $i < sizeOf($hostnames); $i++){

				$names = preg_replace('!\s+!', ' ', $hostnames[$i]);

				$name = explode(" ", $names)[0];

				$adresses = preg_replace('!\s+!', ' ', $ipAdresses[$i]);

				$data = explode(" ", $adresses);

				$ip = $data[0];

				$mac = $data[2];

				array_push($networkDevices, array('id' => $ip, 'title' => $name, 'id2' => $mac));
			}

			return array('devices' => $networkDevices);
		default:
			return "nosuchgateway";
	}
}
'''

    def add_edit_device(self, user, device_id, type, data, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission("admin"):
            return Status(type=STATUS_NO_ADMIN).get_dict()

        add_new = (device_id == None or device_id == "" or device_id == "-1")

        data_array = json.loads(data)

        try:
            #XBOX One WOL
            if type == XBOX_ONE_WOL:
                param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'],
                               'ip': data_array['ip'], 'live_id': data_array['liveid']}

                if device_id == "-1" or device_id is -1:
                    db.insert(
                        "INSERT OR IGNORE INTO 'XBOX_ONE_WOL' (LOCATION, ICON, NAME, IP_ADDRESS, XBOX_LIVE_ID) VALUES (:room, :icon, :name, :ip, :live_id);",
                        param_array)
                else:

                    param_array['id'] = device_id

                    db.update(
                        "UPDATE OR IGNORE 'XBOX_ONE_WOL' SET LOCATION = :room, ICON = :icon, NAME = :name, IP_ADDRESS = :ip, XBOX_LIVE_ID = :live_id WHERE ID == :id;",
                        param_array)

            # Philips Hue
            elif type == PHILIPS_HUE_LIGHT:
                param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'], 'id': device_id}

                db.insert(
                    "INSERT OR IGNORE INTO 'PHILIPS_HUE_LIGHTS' (LOCATION, ID, ICON, NAME) VALUES (:room, :id, :icon, :name);",
                    param_array)

                db.update(
                    "UPDATE OR IGNORE 'PHILIPS_HUE_LIGHTS' SET LOCATION = :room, ICON = :icon, NAME = :name WHERE ID == :id;",
                    param_array)
            #MQTT Sensor
            elif type == MQTT_SENSOR:
                param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data['icon'], 'id': device_id,
                               'key': data_array['key'], 'topic': data_array['topic'], 'save_data': data_array['save_data']}

                db.insert(
                    "INSERT OR IGNORE INTO 'MQTT_SENSORS' (ROOM, ID, ICON, NAME, KEY, TOPIC, SAVE_DATA) VALUES (:room, :id, :icon, :name, :key, :topic, :save_data);",
                    param_array)

                db.update(
                    "UPDATE OR IGNORE 'MQTT_SENSORS' SET ROOM = :room, ICON = :icon, NAME = :name, KEY = :key, TOPIC = :topic, SAVE_DATA = :save_data, WHERE ID == :id;",
                    param_array)

            #433 MHz
            elif type == FUNKSTECKDOSE:
                param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'],
                               'hauscode': data_array['hauscode'], 'nummer': data_array['steckdosennummer']}

                '''if add_new:
                                data = db.select_one("SELECT COUNT(*) FROM 'funksteckdosen' WHERE HAUSCODE = :hauscode AND STECKDOSENNUMMER = :nummer;",
                                            {'hauscode': data_array['hauscode'], 'nummer': data_array['steckdosennummer']})
        
                                count = cur.fetchone()['COUNT(*)']
        
                                if(int(count) != 0):
                                    return "hauscodesocketnumberinuse"
                            else:
                                param_array['id'] = device_id'''

                if add_new:
                    db.insert("INSERT INTO 'funksteckdosen' (ROOM, HAUSCODE, STECKDOSENNUMMER, ICON, NAME) VALUES (:room, :hauscode, :nummer, :icon, :name);",
                                param_array)
                else:
                    param_array['id'] = device_id

                    db.update("UPDATE 'funksteckdosen' SET ROOM = :room, HAUSCODE = :hauscode, STECKDOSENNUMMER = :nummer, ICON = :icon, NAME = :name WHERE DEVICE == :id;",
                                param_array)
            # MAX!
            elif type == MAX_THERMOSTAT:
                param_array = {'room': data_array['room'], 'name': data_array['name'],
                               'icon': data_array['icon'], 'id': device_id}

                db.update("UPDATE OR IGNORE 'MAX_THERMOSTATS' SET RAUM = :room, NAME = :name, ICON = :icon WHERE ID = :id;",
                    param_array)

                db.insert("INSERT OR IGNORE INTO 'MAX_THERMOSTATS' (RAUM, ID, NAME, ICON) VALUES (:room, :id, :name, :icon);",
                    param_array)
            # Rademacher!
            elif type == RADEMACHER_THERMOSTAT:
                param_array = {'room': data_array['room'], 'name': data_array['name'],
                               'icon': data_array['icon'], 'id': device_id}

                db.update("UPDATE OR IGNORE 'HOMEPILOT_THERMOSTATS' SET ROOM = :room, NAME = :name, ICON = :icon WHERE ID = :id;",
                    param_array)

                db.insert("INSERT OR IGNORE INTO 'HOMEPILOT_THERMOSTATS' (ROOM, ID, NAME, ICON) VALUES (:room, :id, :name, :icon);",
                    param_array)
            elif type == RADEMACHER_BLIND_CONTROL:
                param_array = {'room': data_array['room'], 'name': data_array['name'],
                               'icon': data_array['icon'], 'id': device_id}

                db.update("UPDATE OR IGNORE 'HOMEPILOT_BLIND_CONTROL' SET LOCATION = :room, NAME = :name, ICON = :icon WHERE ID = :id;",
                    param_array)

                db.insert("INSERT OR IGNORE INTO 'HOMEPILOT_BLIND_CONTROL' (LOCATION, ID, NAME, ICON) VALUES (:room, :id, :name, :icon);",
                    param_array)

            #Z-Wave
            elif type == ZWAVE_SENSOR:
                param_array = {'room': data_array['room'], 'savedata': data_array['savedata'], 'shortform': data_array['name'],
                               'sensor_type': data_array['sensor_type'], 'icon': data_array['icon'], 'id': device_id}

                count = db.select_one("SELECT COUNT(*) FROM ZWAVE_SENSOREN WHERE ID = :id", {'id': device_id})
                add_new = (count['COUNT(*)'] == 0)

                if add_new:
                    db.insert("INSERT INTO 'ZWAVE_SENSOREN' (RAUM, ID, SHORTFORM, ICON, SAVE_DATA, SENSOR_TYPE) VALUES (:room, :id, :shortform, :icon, :savedata, :sensor_type);",
                                param_array)
                else:
                    db.update("UPDATE 'ZWAVE_SENSOREN' SET RAUM = :room, SHORTFORM = :shortform, ICON = :icon, SAVE_DATA = :savedata, SENSOR_TYPE = :sensor_type WHERE ID = :id;",
                                param_array)
            elif type == ZWAVE_THERMOSTAT:
                param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'], 'id': device_id}

                count = db.select_one("SELECT COUNT(*) FROM ZWAVE_THERMOSTATS WHERE THERMOSTAT_ID = :id", {'id': device_id})
                add_new = (count['COUNT(*)'] == 0)

                if add_new:
                    db.insert("INSERT INTO 'ZWAVE_THERMOSTATS' (RAUM, NAME, THERMOSTAT_ID, ICON) VALUES (:room,:name,:id,:icon);",
                    param_array)
                else:
                    db.update("UPDATE 'ZWAVE_THERMOSTATS' SET RAUM = :room, NAME = :name, THERMOSTAT_ID = :id, ICON =:icon WHERE THERMOSTAT_ID = :id;",
                    param_array)
            elif type == ZWAVE_POWER_METER:
                param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'], 'id': device_id, 'daily_reset': data_array['is_reset_daily']}

                count = db.select_one("SELECT COUNT(*) FROM ZWAVE_POWER_METER WHERE DEVICE_ID = :id", {'id': device_id})
                add_new = (count['COUNT(*)'] == 0)

                if add_new:
                    db.insert("INSERT INTO 'ZWAVE_POWER_METER' (ROOM_ID, DEVICE_ID, DEVICE_NAME, ICON, IS_RESET_DAILY) VALUES (:room,:id,:name,:icon,:daily_reset);",
                    param_array)
                else:
                    db.update("UPDATE 'ZWAVE_POWER_METER' SET ROOM_ID = :room, DEVICE_NAME = :name, DEVICE_ID = :id, ICON =:icon, IS_RESET_DAILY = :daily_reset WHERE DEVICE_ID =:id;",
                    param_array)
            elif type == ZWAVE_SWITCH:
                param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'], 'id': device_id}

                count = db.select_one("SELECT COUNT(*) FROM ZWAVE_SWITCHES WHERE ID = :id", {'id': device_id})
                add_new = (count['COUNT(*)'] == 0)

                if add_new:
                    db.insert("INSERT INTO 'ZWAVE_SWITCHES' (LOCATION, NAME, ID, ICON) VALUES (:room,:name,:id,:icon);",
                    param_array)
                else:
                    db.update("UPDATE 'ZWAVE_SWITCHES' SET LOCATION = :room, NAME = :name, ID = :id, ICON =:icon WHERE ID =:id;",
                    param_array)
            elif type == ZWAVE_DIMMER:
                param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'], 'id': device_id}

                count = db.select_one("SELECT COUNT(*) FROM ZWAVE_DIMMER WHERE ID = :id", {'id': device_id})
                add_new = (count['COUNT(*)'] == 0)

                if add_new:
                    db.insert("INSERT INTO 'ZWAVE_DIMMER' (LOCATION, NAME, ID, ICON) VALUES (:room,:name,:id,:icon);",
                    param_array)
                else:
                    db.update("UPDATE 'ZWAVE_DIMMER' SET LOCATION = :room, NAME = :name, ID = :id, ICON =:icon WHERE ID =:id;",
                    param_array)
            #Media Center
            elif type == MEDIA_CENTER:
                param_array = {'room': data_array['room'], 'name': data_array['name'], 'ip': data_array['ip'],
                               'username': data_array['username'], 'password': data_array['password'], 'type': data_array['type']}

                if add_new:
                    db.insert("INSERT INTO 'MEDIA_CENTER' (LOCATION, NAME, IP, USERNAME, PASSWORD, TYPE) VALUES (:room,:name, :ip, :username, :password, :type);",
                    param_array)
                else:
                    param_array['id'] = device_id
                    db.update("UPDATE 'MEDIA_CENTER' SET LOCATION = :room, NAME = :name, IP = :id, USERNAME = :username, PASSWORD = :password, TYPE = :type, ICON =:icon WHERE ID = :id;",
                    param_array)
            #URL-Devices
            elif type == URL_SWITCH:
                param_array = {'room': data_array['room'], 'name': data_array['name'], 'on': data_array['onurl'],
                               'off': data_array['offurl'], 'val': data_array['valurl'], 'icon': data_array['icon']}

                if add_new:
                    db.insert("INSERT INTO 'URL_SWITCH_BINARY' (LOCATION, NAME, URL_ON, URL_OFF, URL_GET_STATE, ICON) VALUES (:room,:name, :on, :off, :val, :icon);",
                    param_array)
                else:
                    param_array['id'] = device_id
                    db.update("UPDATE 'URL_SWITCH_BINARY' SET LOCATION = :room, NAME = :name, URL_ON = :on, URL_OFF = :off, URL_GET_STATE = :val, ICON =:icon WHERE ID = :id;",
                    param_array)
            elif type == URL_TOGGLE:
                param_array = {'room': data_array['room'], 'name': data_array['name'], 'toggle': data_array['toggleurl'],
                               'icon': data_array['icon']}

                if add_new:
                    db.insert("INSERT INTO 'URL_TOGGLE' (LOCATION, NAME, TOGGLE_URL, ICON) VALUES (:room, :name, :toggle, :icon);",
                    param_array)
                else:
                    param_array['id'] = device_id
                    db.update("UPDATE 'URL_TOGGLE' SET LOCATION = :room, NAME = :name, TOGGLE_URL = :toggle, ICON =:icon WHERE ID = :id;",
                    param_array)
            elif type == URL_RGB_LIGHT:
                param_array = {'room': data_array['room'], 'name': data_array['name'], 'url': data_array['url'],
                               'icon': data_array['icon']}

                if add_new:
                    db.insert("INSERT INTO 'URL_RGB_LIGHT' (LOCATION, NAME, URL, ICON) VALUES (:room, :name, :url, :icon);",
                    param_array)
                else:
                    param_array['id'] = device_id
                    db.update("UPDATE 'URL_RGB_LIGHT' SET LOCATION = :room, NAME = :name, URL = :toggle, ICON =:icon WHERE ID = :id;",
                    param_array)
            #MQTT
            elif type == MQTT_TRIGGER:
                param_array = {'room': data_array['room'], 'name': data_array['name'], 'topic': data_array['topic'],
                               'icon': data_array['icon'], 'type': data_array['type']}

                if add_new:
                    db.insert("INSERT INTO 'MQTT_TRIGGERS' (LOCATION, NAME, TOPIC, TYPE, ICON) VALUES (:room, :name, :topic, :type, :icon);",
                    param_array)
                else:
                    param_array['id'] = device_id
                    db.update("UPDATE 'MQTT_TRIGGERS' SET LOCATION = :room, NAME = :name, TOPIC = :topic, ICON =:icon WHERE ID = :id;",
                    param_array)
            elif type == MQTT_SENSOR:
                param_array = {'room': data_array['room'], 'name': data_array['name'], 'topic': data_array['topic'],
                               'icon': data_array['icon'], 'type': data_array['type']}

                if add_new:
                    db.insert("INSERT INTO 'MQTT_SENSORS' (LOCATION, NAME, TOPIC, TYPE, ICON) VALUES (:room, :name, :topic, :type, :icon);",
                    param_array)
                else:
                    param_array['id'] = device_id
                    db.update("UPDATE 'MQTT_SENSORS' SET LOCATION = :room, NAME = :name, TOPIC = :topic, ICON =:icon WHERE ID = :id;",
                    param_array)
            #Wake on Lan
            elif type == WAKE_ON_LAN:
                param_array = {'room': data_array['room'], 'name': data_array['name'],
                               'icon': data_array['icon'], 'mac': data_array['mac']}

                if add_new:
                    db.insert("INSERT INTO 'WAKE_ON_LAN' (LOCATION, NAME, ICON, MAC_ADDRESS) VALUES (:room, :name, :icon, :mac);",
                    param_array)
                else:
                    param_array['id'] = device_id
                    db.update("UPDATE 'WAKE_ON_LAN' SET LOCATION = :room, NAME = :name, MAC_ADDRESS = :mac, ICON =:icon WHERE DEVICE = :id;",
                    param_array)
            else:
                return {'result': 'nosuchtype'}

            return Status(type=STATUS_OK).get_dict()
        except:
            return Status(type=STATUS_ERROR).get_dict()

    def delete_device(self, user, type, id, db: Database = None):
        if db is None:
            db = Database()
        #type = type.encode('utf-8')
        db_cols = {
            FUNKSTECKDOSE: {'table': 'FUNKSTECKDOSEN', 'location': 'ROOM', 'id': 'DEVICE'},
            PHILIPS_HUE_LIGHT: {'table': 'PHILIPS_HUE_LIGHTS', 'location': 'LOCATION', 'id': 'ID'},
            WAKE_ON_LAN: {'table': 'WAKE_ON_LAN', 'location': 'LOCATION', 'id': 'DEVICE'},
            XBOX_ONE_WOL: {'table': 'XBOX_ONE_WOL', 'location': 'LOCATION', 'id': 'ID'},
            URL_RGB_LIGHT: {'table': 'URL_RGB_LIGHT', 'location': 'LOCATION', 'id': 'ID'},
            MQTT_SENSOR: {'table': 'MQTT_SENSORS', 'location': 'ROOM', 'id': 'ID'},
            URL_TOGGLE: {'table': 'URL_TOGGLE', 'location': 'LOCATION', 'id': 'ID'},
            URL_SWITCH: {'table': 'URL_SWITCH', 'location': 'LOCATION', 'id': 'ID'},
            ZWAVE_DIMMER: {'table': 'ZWAVE_DIMMER', 'location': 'LOCATION', 'id': 'ID'},
            ZWAVE_SWITCH: {'table': 'ZWAVE_SWITCHES', 'location': 'LOCATION', 'id': 'ID'},
            ZWAVE_SENSOR: {'table': 'ZWAVE_SENSOREN', 'location': 'RAUM', 'id': 'ID'},
            ZWAVE_POWER_METER: {'table': 'ZWAVE_POWER_METER', 'location': 'ROOM_ID', 'id': 'DEVICE_ID'},
            ZWAVE_THERMOSTAT: {'table': 'ZWAVE_THERMOSTATS', 'location': 'RAUM', 'id': 'THERMOSTAT_ID'},
            MAX_THERMOSTAT: {'table': 'MAX_THERMOSTATS', 'location': 'RAUM', 'id': 'ID'},
            RADEMACHER_THERMOSTAT: {'table': 'HOMEPILOT_THERMOSTATS', 'location': 'ROOM', 'id': 'ID'},
            RADEMACHER_BLIND_CONTROL: {'table': 'HOMEPILOT_BLIND_CONTROL', 'location': 'LOCATION', 'id': 'ID'}
        }

        if type not in db_cols:
            return {'result': 'nosuchtype'}

        data = db.select_one("SELECT * FROM "+db_cols[type]['table']+" WHERE "+db_cols[type]['id']+" = :id",
                    {'id': id})

        Logger.log(data)

        if not user.has_permission(data[db_cols[type]['location']]):
            return {'result': 'nopermission'}

        db.delete("DELETE FROM "+db_cols[type]['table']+" WHERE " + db_cols[type]['id'] + " = :id",
                    {'id': id})

        return Status(type=STATUS_OK).get_dict()

    def get_device_data(self, user, type, id, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission("admin"):
            return {'result': 'noadmin'}

        room_array = {}

        for room in RoomManager().get_rooms(user, db):
            room_array[room.id] = room.name

        Logger.log(room_array)

        param_array = {'id': id}

        device_data = {}

        #Funksteckdose
        if type == FUNKSTECKDOSE:
            data = db.select_one("SELECT * FROM FUNKSTECKDOSEN WHERE DEVICE = :id", param_array)

            device_data = {'room': room_array[int(data['ROOM'])], 'location': data['ROOM'], 'name': data['NAME'],
                           'hauscode': data['HAUSCODE'], 'icon': data['ICON'], 'steckdosennummer': data['STECKDOSENNUMMER']}
        #Z-Wave
        elif type == ZWAVE_POWER_METER:
            data = db.select_one("SELECT * FROM ZWAVE_POWER_METER WHERE DEVICE_ID = :id", param_array)

            device_data = {'room': room_array[int(data['ROOM_ID'])], 'location': data['ROOM_ID'], 'name': data['DEVICE_NAME'],
                           'is_reset_daily': (data['IS_RESET_DAILY']==1), 'icon': data['ICON']}
        elif type == ZWAVE_SENSOR:
            data = db.select_one("SELECT * FROM ZWAVE_SENSOREN WHERE ID = :id", param_array)

            device_data = {'room': room_array[int(data['RAUM'])], 'location': data['RAUM'], 'name': data['SHORTFORM'],
                           'save_data': data['SAVE_DATA'], 'icon': data['ICON'],
                           'sensor_type': data['SENSOR_TYPE'], 'sensor_type_name': SENSOR_TYPE_MAP[data['SENSOR_TYPE']]['name']}
        elif type == ZWAVE_THERMOSTAT:
            data = db.select_one("SELECT * FROM ZWAVE_THERMOSTATS WHERE THERMOSTAT_ID = :id", param_array)

            device_data = {'room': room_array[int(data['RAUM'])], 'location': data['RAUM'], 'name': data['NAME'],
                           'icon': data['ICON']}
        elif type == ZWAVE_SWITCH:
            data = db.select_one("SELECT * FROM ZWAVE_SWITCHES WHERE ID = :id", param_array)

            device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                           'icon': data['ICON']}
        elif type == ZWAVE_DIMMER:
            data = db.select_one("SELECT * FROM ZWAVE_DIMMER WHERE ID = :id", param_array)

            device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                           'icon': data['ICON']}
        #MQTT-Sensor
        elif type == MQTT_SENSOR:
            data = db.select_one("SELECT * FROM MQTT_SENSORS, MQTT_DEVICES WHERE MQTT_SENSORS.DEVICE_ID = MQTT_DEVICES.ID AND MQTT_SENSORS.ID = :id", param_array)

            device_data = {'room': room_array[int(data['ROOM'])], 'location': data['ROOM'], 'name': data['NAME'], 'icon': data['ICON'],
                           'topic': data['TOPIC'], 'key': data['KEY'], 'sensor_type': data['TYPE'],
                           'sensor_type_name': SENSOR_TYPE_MAP[data['TYPE']]['name'], 'save_data': data['SAVE_DATA']==1}
        #IP Kamera
        elif type == IP_CAM:
            data = db.select_one("SELECT * FROM ZWAVE_DIMMER WHERE ID = :id", param_array)

            device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                           'icon': data['ICON'], 'ip': data['IP'], 'port': data['PORT'], 'path': data['PATH'],
                           'username': data['USERNAME'], 'password': data['PASSWORD'], 'recordfootage': data['RECORD_FOOTAGE'],
                           'autodelete': data['AUTO_DELETE'], 'detectionthreshold': data['MOTION_DETECTION_THRESHOLD']}
        #URL-Toggle
        elif type == URL_TOGGLE:
            data = db.select_one("SELECT * FROM URL_TOGGLE WHERE ID = :id", param_array)

            device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                           'icon': data['ICON'], 'toggle_url': data['TOGGLE_URL']}
        #URL-Switch
        elif type == URL_SWITCH:
            data = db.select_one("SELECT * FROM URL_SWITCH_BINARY WHERE ID = :id", param_array)

            device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                           'icon': data['ICON'], 'on_url': data['URL_ON'], 'off_url': data['URL_OFF'],
                           'get_state_url': data['URL_GET_STATE']}
        #URL RGB-Licht
        elif type == URL_RGB_LIGHT:
            data = db.select_one("SELECT * FROM URL_RGB_LIGHT WHERE ID = :id", param_array)

            device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                           'icon': data['ICON'], 'url': data['URL']}
        #Philips Hue RGB-Licht
        elif type == PHILIPS_HUE_LIGHT:
            data = db.select_one("SELECT * FROM PHILIPS_HUE_LIGHTS WHERE ID = :id", param_array)

            device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                           'icon': data['ICON'], 'type': data['TYPE']}
        #MAX Thermostat
        elif type == MAX_THERMOSTAT:
            data = db.select_one("SELECT * FROM MAX_THERMOSTATS WHERE ID = :id", param_array)

            device_data = {'room': room_array[int(data['RAUM'])], 'location': data['RAUM'], 'name': data['NAME'],
                           'icon': data['ICON']}
        #Wake on Lan
        elif type == WAKE_ON_LAN:
            data = db.select_one("SELECT * FROM WAKE_ON_LAN WHERE DEVICE = :id", param_array)

            device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                           'icon': data['ICON'], 'mac': data['MAC_ADDRESS']}
        #XBOX Wake on Lan
        elif type == XBOX_ONE_WOL:
            data = db.select_one("SELECT * FROM XBOX_ONE_WOL WHERE ID = :id", param_array)

            device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                           'icon': data['ICON'], 'ip': data['IP_ADDRESS'], 'liveid': data['XBOX_LIVE_ID']}
        #Rademacher
        elif type == RADEMACHER_THERMOSTAT:
            data = db.select_one("SELECT * FROM HOMEPILOT_THERMOSTATS WHERE ID = :id", param_array)

            device_data = {'room': room_array[int(data['ROOM'])], 'location': data['ROOM'], 'name': data['NAME'],
                           'icon': data['ICON']}
        elif type == RADEMACHER_BLIND_CONTROL:
            data = db.select_one("SELECT * FROM HOMEPILOT_BLIND_CONTROL WHERE ID = :id", param_array)

            device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                           'icon': data['ICON']}
        else:
            return {'result': 'nosuchtype'}

        return device_data


from Homevee.AIAgents.ImageClassifier.PeopleClassifier import PeopleClassifier
from Homevee.AIAgents.ImageClassifierRetrainer.PeopleClassifierRetrainer import PeopleClassifierRetrainer
from Homevee.Manager.ControlManager.ImageClassifierManager import ImageClassifierManager


class PeopleClassifierManager(ImageClassifierManager):
    def __init__(self):
        data_table = "PEOPLE_DATA"
        learning_data_table = "PEOPLE_LEARNING_DATA"
        settings_prefix = "AR_"

        classifier = PeopleClassifier()
        retrainer = PeopleClassifierRetrainer()

        is_training_tag = "PEOPLE_CLASSIFIER_TRAINING_RUNNING"
        classifier_tag = "people"

        super(PeopleClassifierManager, self).__init__(data_table=data_table, learning_data_table=learning_data_table,
                                                      settings_prefix=settings_prefix, is_training_tag=is_training_tag,
                                                      classifier_tag=classifier_tag, classifier=classifier, retrainer=retrainer)
        return


from Homevee.APIModule import APIModule
from Homevee.Item.Room import Room
from Homevee.Item.Status import *
from Homevee.Manager.RoomManager import RoomManager

ACTION_KEY_GET_ROOMS = "getrooms"
ACTION_KEY_ADD_EDIT_ROOM = "addeditroom"
ACTION_KEY_DELETE_ROOM = "deleteroom"
ACTION_KEY_MOVE_ITEMS_AND_DELETE_OLD_ROOM = "moveitemsanddeleteoldroom"
ACTION_KEY_DELETE_ROOM_WITH_ITEMS = "deleteroomwithitems"

class RoomAPIModule(APIModule):
    def __init__(self):
        super(RoomAPIModule, self).__init__()
        self.room_manager = RoomManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_ROOMS: self.get_rooms,
            ACTION_KEY_ADD_EDIT_ROOM: self.add_edit_room,
            ACTION_KEY_DELETE_ROOM: self.delete_room,
            ACTION_KEY_MOVE_ITEMS_AND_DELETE_OLD_ROOM: self.move_items_and_delete_old_room,
            ACTION_KEY_DELETE_ROOM_WITH_ITEMS: self.delete_room_with_items
        }
        return mappings

    def get_rooms(self, user, request, db) -> Status:
        rooms = self.room_manager.get_rooms(user, db)
        return Status(type=STATUS_OK, data={'rooms': Room.list_to_dict(rooms)})

    def add_edit_room(self, user, request, db) -> Status:
        self.room_manager.add_edit_room(user, request['roomname'], request['location'], request['icon'], db)
        return Status(type=STATUS_OK)

    def delete_room(self, user, request, db) -> Status:
        self.room_manager.delete_room(user, request['location'], db)
        return Status(type=STATUS_OK)

    def move_items_and_delete_old_room(self, user, request, db) -> Status:
        self.room_manager.move_items_and_delete_old_room(user, request['oldroom'], request['newroom'], db)
        return Status(type=STATUS_OK)

    def delete_room_with_items(self, user, request, db) -> Status:
        self.room_manager.delete_room_with_items(user, request['location'], db)
        return Status(type=STATUS_OK)


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.SmartSpeakerManager import SmartSpeakerManager

ACTION_KEY_GET_SMART_SPEAKERS = "getsmartspeakers"

class SmartSpeakerAPIModule(APIModule):
    def __init__(self):
        super(SmartSpeakerAPIModule, self).__init__()
        self.manager = SmartSpeakerManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_SMART_SPEAKERS: self.get_smart_speakers
        }

        return mappings

    def get_smart_speakers(self, user, request, db):
        data = self.manager.get_smart_speakers(user, db)
        return Status(type=STATUS_OK, data={'speakers':data})


#!/usr/bin/python
# -*- coding: utf-8 -*-
import random
import urllib.error
import urllib.error
import urllib.parse
import urllib.parse
import urllib.request
import urllib.request

from Homevee.Helper import Logger
from Homevee.Utils.Database import Database
from Homevee.VoiceAssistant import Helper
from Homevee.VoiceAssistant.Modules import VoiceModule


class VoiceGetActivitiesModule(VoiceModule):
    def get_pattern(self, db: Database = None):
        if db is None:
            db = Database()
        return [
            [['mir', 'uns'], 'ist', 'langweilig'],
            ['was', ['kann', 'können'], ['ich', 'wir'], ['tun', 'machen', 'unternehmen']]
        ]

    def get_label(self):
        return "activity"

    def run_command(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return self.get_activities(username, text, context, db)

    def get_activities(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        try:
            url = Helper.SMART_API_PATH + "/?action=activity&text=" + urllib.parse.quote(text.encode('utf8'))
            Logger.log(url)

            response = urllib.request.urlopen(url)
            data = response.read()

            data = data.decode('utf-8')

            if data is not None:
                return {'msg_speech': data, 'msg_text': data}
            else:
                output = self.get_error()
        except urllib.error.HTTPError as e:
            output = self.get_error()

        return {'msg_speech': output, 'msg_text': output}

    def get_error(self):
        return random.choice([
            'Mir fällt gerade nichts ein, was du tun könntest.'
        ])


#!/usr/bin/python
# -*- coding: utf-8 -*-
from sqlite3.dbapi2 import Row
from typing import Type, List

from Homevee.Exception import NoPermissionException, RoomHasItemsException
from Homevee.Item import Item
from Homevee.Item.Room import Room
from Homevee.Item.Status import *
from Homevee.Item.User import User
from Homevee.Manager import Manager
from Homevee.Manager.SceneManager import SceneManager
from Homevee.Utils.Database import Database


class RoomManager(Manager):
    def __init__(self):
        super(RoomManager, self).__init__()
        return

    def item_exists(self, item: Room, db: Database) -> bool:
        return self.find_by_id(item.id, db) is not None

    def create(self, item: Room, db: Database) -> bool:
        return (db.insert("INSERT INTO ROOMS (NAME, ICON) VALUES (:name, :icon)",
                  {'name': item.name, 'icon': item.icon}) != False)

    def update(self, item: Room, db: Database) -> bool:
        return db.update("UPDATE ROOMS SET NAME = :name, ICON = :icon WHERE LOCATION = :id",
                            {'name': item.name, 'icon': item.icon, 'id': item.id})

    def delete(self, item: Room, db: Database) -> bool:
        # TODO implement room deletion
        return False

    def get_all(self, db: Database) -> List[Type[Item]]:
        return self.find_by_query('SELECT * FROM ROOMS', {}, db)

    def find_by_id(self, id: int, db: Database) -> Type[Item]:
        return self.find_one_by_query('SELECT * FROM ROOMS WHERE ID = :id', {'id': id}, db)

    def construct_item_from_dict(self, dict: dict) -> Type[Item]:
        return Room.create_from_dict(dict)

    def check_permission(self, user: User, item: Room) -> bool:
        return user.has_permission(item.id)

    def create_item_from_db_result(self, result: Row) -> Room:
        return Room(result['NAME'], result['ICON'], result['LOCATION'])

    def get_rooms(self, user: User, db: Database = None) -> list:
        if db is None:
            db = Database()
        rooms = []
        all_rooms = Room.load_all(db)
        for room in all_rooms:
            if(user.has_permission(room.id)):
                rooms.append(room)
        return rooms

    def add_edit_room(self, user, room_name, room_key, icon, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission("admin"):
            raise NoPermissionException()

        try:
            room = Item.load_from_db(Room, room_key, db)
            room.name = room_name
            room.icon = icon
        except:
            room = Room(room_name, room_key)

        return room.save()

    def delete_room(self, user, room_key, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission("admin"):
            raise NoPermissionException()

        room = Item.load_from_db(Room, room_key, db)

        roomdata = room.get_room_data(db)

        #roomdata = get_room_data(user, room_key, db)['roomdata']

        if roomdata is not None and len(roomdata) == 1:
            if roomdata[0]['type'] == "scenes":
                scenes = SceneManager().get_scenes(user, room_key, db)

                for scene in scenes:
                    if scene['room'] == room_key:
                        return Status(type=STATUS_ROOM_HAS_ITEMS).get_dict()
        elif roomdata is not None and len(roomdata) > 1:
            raise RoomHasItemsException()

        return room.delete()

    def move_items_and_delete_old_room(self, user, old_room, new_room, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission("admin"):
            return Status(type=STATUS_NO_ADMIN).get_dict()

        params = {'oldroom': old_room, 'newroom': new_room}

        query_array = []

        #Funksteckdosen
        query_array.append("UPDATE 'funksteckdosen' SET ROOM = :newroom WHERE ROOM == :oldroom;")

        #Z-Wave
        query_array.append("UPDATE 'ZWAVE_SENSOREN' SET RAUM = :newroom WHERE RAUM == :oldroom;")
        query_array.append("UPDATE 'ZWAVE_THERMOSTATS' SET RAUM = :newroom WHERE RAUM == :oldroom;")

        #DIY
        query_array.append("UPDATE 'DIY_DEVICES' SET ROOM = :newroom WHERE ROOM == :oldroom;")
        query_array.append("UPDATE 'DIY_SENSORS' SET RAUM = :newroom WHERE RAUM == :oldroom;")
        query_array.append("UPDATE 'DIY_REEDSENSORS' SET RAUM = :newroom WHERE RAUM == :oldroom;")
        query_array.append("UPDATE 'DIY_SWITCHES' SET RAUM = :newroom WHERE RAUM == :oldroom;")

        #Szenen
        query_array.append("UPDATE 'SCENES' SET ROOM = :newroom WHERE ROOM == :oldroom;")


        for query in query_array:
            db.update(query, params)

        return self.delete_room(user, old_room, db)

    def delete_room_with_items(self, user, room_key, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission("admin"):
            return Status(type=STATUS_NO_ADMIN).get_dict()

        params = {'oldroom': room_key}

        query_array = []

        # Funksteckdosen
        query_array.append("DELETE FROM 'funksteckdosen' WHERE ROOM == :oldroom;")

        # Z-Wave
        query_array.append("DELETE FROM 'ZWAVE_SENSOREN' WHERE RAUM == :oldroom;")
        query_array.append("DELETE FROM 'ZWAVE_THERMOSTATS' WHERE RAUM == :oldroom;")

        # DIY
        query_array.append("DELETE FROM 'DIY_DEVICES' WHERE ROOM == :oldroom;")
        query_array.append("DELETE FROM 'DIY_REEDSENSORS' WHERE RAUM == :oldroom;")
        query_array.append("DELETE FROM 'DIY_SENSORS' WHERE RAUM == :oldroom;")
        query_array.append("DELETE FROM 'DIY_SWITCHES' WHERE RAUM == :oldroom;")

        # Szenen
        query_array.append("DELETE FROM 'SCENES' WHERE ROOM == :oldroom;")


        for query in query_array:
            db.delete(query, params)

        return self.delete_room(user, room_key, db)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

from Homevee.Manager.ControlManager.ActionManager import ActionManager
from Homevee.Utils.Database import Database


class CustomVoiceCommandManager():
    def __init__(self):
        return

    def get_voice_commands(self, db):
        commands = []
        results = db.select_all("SELECT * FROM CUSTOM_VOICE_COMMANDS", {})

        for item in results:
            command_data = self.get_command_data(item['ID'], db)

            responses = self.get_response_data(item['ID'], db)

            commands.append({'id': item['ID'], 'name': item['NAME'], 'action_data': item['ACTION_DATA'],
                          'command_data': command_data,
                             'response_data': responses})

        return commands

    def get_command_data(self, id, db: Database = None):
        if db is None:
            db = Database()
        items = db.select_all("SELECT * FROM CUSTOM_VOICE_COMMAND_SENTENCES WHERE COMMAND_ID = :id", {'id': id})

        data = []

        for item in items:
            data.append(item['COMMAND'])

        return data

    def get_response_data(self, id, db: Database = None):
        if db is None:
            db = Database()
        items = db.select_all("SELECT * FROM CUSTOM_VOICE_COMMAND_RESPONSES WHERE COMMAND_ID = :id", {'id': id})

        data = []

        for item in items:
            data.append(item['RESPONSE'])

        return data

    def add_edit_voice_command(self, username, id, name, command_data, response_data, action_data, db: Database = None):
        if db is None:
            db = Database()
        add_new = (id == None or id == "" or id == "-1")

        if(add_new):
            id = db.insert("INSERT INTO CUSTOM_VOICE_COMMANDS (NAME, ACTION_DATA) VALUES (:name, :actions)",
                        {'name': name, 'actions': action_data})

            command_data = json.loads(command_data)
            self.add_command_data(command_data, id, db)

            response_data = json.loads(response_data)
            self.add_response_data(response_data, id, db)
        else:
            db.update("UPDATE AUTOMATION_DATA SET NAME = :name, ACTION_DATA = :actions WHERE ID = :id",
                {'name': name, 'actions': action_data, 'id': id})

            db.delete("DELETE FROM CUSTOM_VOICE_COMMAND_SENTENCES WHERE COMMAND_ID = :id", {'id': id})
            db.delete("DELETE FROM CUSTOM_VOICE_COMMAND_RESPONSES WHERE COMMAND_ID = :id", {'id': id})

            command_data = json.loads(command_data)
            self.add_command_data(command_data, id, db)

            response_data = json.loads(response_data)
            self.add_response_data(response_data, id, db)

    def delete_voice_command(self, username, id, db: Database = None):
        if db is None:
            db = Database()
        db.delete("DELETE FROM CUSTOM_VOICE_COMMANDS WHERE ID = :id", {'id': id})
        db.delete("DELETE FROM CUSTOM_VOICE_COMMAND_RESPONSES WHERE COMMAND_ID = :id", {'id': id})
        db.delete("DELETE FROM CUSTOM_VOICE_COMMAND_SENTENCES WHERE COMMAND_ID = :id", {'id': id})

    def add_command_data(self, commands, id, db: Database = None):
        if db is None:
            db = Database()
        for command in commands:
            param_array = {'id': id, 'command': command.lower()}

            db.insert("""INSERT INTO CUSTOM_VOICE_COMMAND_SENTENCES (COMMAND_ID, COMMAND) VALUES (:id, :command)""",
                param_array)

    def add_response_data(self, responses, id, db: Database = None):
        if db is None:
            db = Database()
        for response in responses:
            param_array = {'id': id, 'response': response}

            db.insert("""INSERT INTO CUSTOM_VOICE_COMMAND_RESPONSES (COMMAND_ID, RESPONSE) VALUES (:id, :response)""",
                param_array)

    def run_custom_voice_commands(self, text, username, db: Database = None):
        if db is None:
            db = Database()
        result = db.select_one("SELECT * FROM CUSTOM_VOICE_COMMAND_SENTENCES, CUSTOM_VOICE_COMMANDS WHERE ID = COMMAND_ID AND COMMAND = :command",
                    {'command': text})

        if(result is None):
            return None

        id = result['ID']

        action_data = result['ACTION_DATA']

        action_data = json.loads(action_data)

        #run actions
        ActionManager().run_actions(action_data, db)

        result = db.select_one("SELECT * FROM CUSTOM_VOICE_COMMAND_RESPONSES WHERE COMMAND_ID = :id ORDER BY RANDOM()",
                    {'id': id})

        return result['RESPONSE']


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.TVPlanManager import TVPlanManager

ACTION_KEY_SET_TV_CHANNELS = "settvchannels"
ACTION_KEY_GET_TV_PLAN = "gettvplan"
ACTION_KEY_GET_ALL_TV_CHANNELS = "getalltvchannels"

class TVPlanAPIModule(APIModule):
    def __init__(self):
        super(TVPlanAPIModule, self).__init__()
        self.tv_manager = TVPlanManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_SET_TV_CHANNELS: self.set_tv_channels,
            ACTION_KEY_GET_TV_PLAN: self.get_tv_plan,
            ACTION_KEY_GET_ALL_TV_CHANNELS: self.get_all_tv_channels
        }

        return mappings

    def set_tv_channels(self, user, request, db):
        data = self.tv_manager.set_tv_channels(user, request['channels'], db)
        return Status(type=STATUS_OK)

    def get_tv_plan(self, user, request, db):
        data = self.tv_manager.get_tv_plan(user, request['time'], db)
        return Status(type=STATUS_OK, data={'data': data})

    def get_all_tv_channels(self, user, request, db):
        data = self.tv_manager.get_all_tv_channels(user, db)
        return Status(type=STATUS_OK, data={'channels': data})


import datetime

import psutil

from Homevee.Utils import Constants


class SystemInfoManager:
    def __init__(self):
        return

    def get_avg_cpu_temp(self):
        temps = psutil.sensors_temperatures(fahrenheit=False)['coretemp']
        temp_sum = 0
        for temp in temps:
            temp_sum += temp.current
        return temp_sum/len(temps)

    def get_free_space_data(self):
        free_space = psutil.disk_usage(Constants.HOMEVEE_DIR).free
        division_counter = 0
        label_division_map = {
            0: "Byte",
            1: "kB",
            2: "MB",
            3: "GB",
            4: "TB"
        }

        free_space_label = "Byte"

        while True:
            if division_counter+1 not in label_division_map or free_space < 1024:
                break

            free_space = free_space/1024
            division_counter += 1
            free_space_label = label_division_map[division_counter]

        return round(free_space, 2), free_space_label

    def get_system_info(self):
        """
        Gets the current system data
        (e.g. uptime, free memory, cpu usage etc.)
        :return:
        """
        systeminfo = []

        # Remote-ID
        #systeminfo.append({'name': 'Remote-ID', 'type': 'remote', 'value': db.get_server_data("REMOTE_ID", Database())})

        #Laufzeit
        boot_time = datetime.datetime.fromtimestamp(psutil.boot_time())
        uptime = boot_time.strftime("%H:%M %d.%m.%Y")
        systeminfo.append({'name': 'Aktiv seit', 'type': 'uptime', 'value': uptime})

        try:
            #CPU-Temperatur
            cpu_temp = self.get_avg_cpu_temp()
            systeminfo.append({'name': 'CPU-Temp', 'type': 'cputemp', 'value': str(cpu_temp)+" &deg;"})
        except:
            #only compatible with linux for now
            pass

        #CPU-Auslastung
        cpu_usage = psutil.cpu_percent(interval=None)
        systeminfo.append({'name': 'CPU-Auslastung', 'type': 'cpuusage', 'value': str(cpu_usage)+" %"})

        #CPU-Kerne
        cpu_cores = psutil.cpu_count(logical=False)
        cpu_threads = psutil.cpu_count()
        systeminfo.append({'name': 'CPU-Kerne', 'type': 'cpucores', 'value': str(cpu_cores)+" Kerne\n("+str(cpu_threads)+" Threads)"})

        #CPU-Takt
        cpu_freq = psutil.cpu_freq().current
        cpu_freq = round(cpu_freq/1000, 2)
        systeminfo.append({'name': 'CPU-Takt', 'type': 'cpufrequency', 'value': str(cpu_freq)+" GHz"})

        #Kernel-Version
        systeminfo.append({'name': 'Version', 'type': 'version', 'value': Constants.HOMEVEE_VERSION_NUMBER})

        #Speichernutzung
        space_usage = psutil.disk_usage(Constants.HOMEVEE_DIR).percent
        systeminfo.append({'name': 'Speichernutzung', 'type': 'spaceusage', 'value': str(space_usage)+" %"})

        #Freier Speicher
        free_space, free_space_label = self.get_free_space_data()
        systeminfo.append({'name': 'Freier Speicher', 'type': 'freespace', 'value': str(free_space)+" "+free_space_label})

        #RAM-Auslastung
        ram_usage = psutil.virtual_memory().percent
        systeminfo.append({'name': 'RAM-Nutzung', 'type': 'ramusage', 'value': str(ram_usage)+" %"})

        return systeminfo


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.ControlManager.WakeOnLanManager import WakeOnLanManager

ACTION_KEY_WAKE_ON_LAN = "wakeonlan"
ACTION_KEY_START_XBOX_ONE = "startxboxone"

class WakeOnLanAPIModule(APIModule):
    def __init__(self):
        super(WakeOnLanAPIModule, self).__init__()
        self.wol_manager = WakeOnLanManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_WAKE_ON_LAN: self.wake_on_lan,
            ACTION_KEY_START_XBOX_ONE: self.start_xbox_one
        }

        return mappings

    def wake_on_lan(self, user, request, db):
        self.wol_manager.wake_on_lan(user, request['id'], db)
        return Status(type=STATUS_OK)

    def start_xbox_one(self, user, request, db):
        self.wol_manager.wake_xbox_on_lan(user, request['id'], db)
        return Status(type=STATUS_OK)


'''
        elif action == "gethomebudgetdata":
            response = HomeBudgetManager().get_home_budget_data(user, request['startdate'], db)
        elif action == "gethomebudgetdatadayitems":
            response = HomeBudgetManager().get_home_budget_data_day_items(user, request['date'], db)
        elif action == "gethomebudgetdatagraph":
            response = HomeBudgetManager().get_home_budget_data_graph(user, request['startdate'], request['enddate'],
                                                                      db)
        elif action == "addedithomebudgetdata":
            response = HomeBudgetManager().add_edit_home_budget_data(user, request['id'], request['date'],
                                                                     request['info'], request['amount'], db)
        elif action == "deletehomebudgetdata":
            response = HomeBudgetManager().delete_home_budget_data(user, request['id'], db)
'''

from Homevee.APIModule import APIModule
from Homevee.Exception import NoPermissionException
from Homevee.Item.Status import *
from Homevee.Manager.HomeBudgetManager import HomeBudgetManager

ACTION_KEY_GET_HOME_BUDGET_DATA = "gethomebudgetdata"
ACTION_KEY_GET_HOME_BUDGET_DATA_DAY_ITEMS = "gethomebudgetdatadayitems"
ACTION_KEY_GET_HOME_BUDGET_DATA_GRAPH = "gethomebudgetdatagraph"
ACTION_KEY_ADD_EDIT_HOME_BUDGET_DATA = "addedithomebudgetdata"
ACTION_KEY_DELETE_HOME_BUDGET_DATA = "deletehomebudgetdata"

class HomeBudgetAPIModule(APIModule):
    def __init__(self):
        super(HomeBudgetAPIModule, self).__init__()
        self.home_budget_manager = HomeBudgetManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_HOME_BUDGET_DATA: self.get_home_budget_data,
            ACTION_KEY_GET_HOME_BUDGET_DATA_DAY_ITEMS: self.get_home_budget_data_items,
            ACTION_KEY_GET_HOME_BUDGET_DATA_GRAPH: self.get_home_budget_data_graph,
            ACTION_KEY_ADD_EDIT_HOME_BUDGET_DATA: self.add_edit_home_budget_data,
            ACTION_KEY_DELETE_HOME_BUDGET_DATA: self.delete_home_budget_data
        }

        return mappings

    def get_home_budget_data(self, user, request, db):
        data = self.home_budget_manager.get_home_budget_data(user, request['startdate'], db)
        return Status(type=STATUS_OK, data=data)

    def get_home_budget_data_items(self, user, request, db):
        data = self.home_budget_manager.get_home_budget_data_day_items(user, request['date'], db)
        return Status(type=STATUS_OK, data={'items':data})

    def get_home_budget_data_graph(self, user, request, db):
        data = self.home_budget_manager.get_home_budget_data_graph(user, request['startdate'], request['enddate'], db)
        return Status(type=STATUS_OK, data=data)

    def add_edit_home_budget_data(self, user, request, db):
        try:
            self.home_budget_manager.add_edit_home_budget_data(user, request['id'],
                            request['date'], request['info'], request['amount'], db)
            return Status(type=STATUS_OK)
        except(NoPermissionException):
            return Status(type=STATUS_NO_PERMISSION)
        except:
            return Status(type=STATUS_ERROR)

    def delete_home_budget_data(self, user, request, db):
        try:
            self.home_budget_manager.delete_home_budget_data(user, request['id'], db)
            return Status(type=STATUS_OK)
        except(NoPermissionException):
            return Status(type=STATUS_NO_PERMISSION)
        except:
            return Status(type=STATUS_ERROR)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime

from Homevee.Cronjob import IntervalCronjob
from Homevee.Helper.HomeveeAPI import HomeveeAPI
from Homevee.Manager.APIKeyManager import APIKeyManager
from Homevee.Utils.Database import Database


class WeatherUpdaterCronjob(IntervalCronjob):
    def __init__(self):
        super(WeatherUpdaterCronjob, self).__init__(task_name="WeatherUpdaterCronjob", interval_seconds=15*60)

    def task_to_do(self, *args):
        self.refresh_weather_cache()

    def get_seconds_to_wait(self, execution_time=None):
        t = datetime.datetime.today()

        seconds_to_wait = (15 * 60) - ((t.minute * 60) - t.second) % 15 * 60

        return seconds_to_wait

    def refresh_weather_cache(self):
        db = Database()

        api_key = APIKeyManager().get_api_key("Open Weather Map", db)
        location_id = db.get_server_data("WEATHER_LOCATION_ID")

        try:
            #url = "http://api.openweathermap.org/data/2.5/forecast/daily?id=" + location_id + "&cnt=16&units=metric&lang=de&type=accurate&APPID=" + api_key
            #response = urllib.request.urlopen(url).read()

            response = HomeveeAPI().get_weather(location_id)

            if(response.status_code == 200):
                data = response.response.encode('utf-8')
            else:
                data = "{}"

                # Wetter-Daten in Datenbank schreiben
            db.insert("INSERT OR REPLACE INTO SERVER_DATA (KEY, VALUE) values('WEATHER_CACHE', :response);",
                        {"response": data})

            return True
        except:
            return False


#!flask/bin/python
import json

from flask import Blueprint, request

from Homevee.API import API
from Homevee.Utils.Database import Database

ProcessDataAPI = Blueprint('ProcessDataAPI', __name__, template_folder='templates')

@ProcessDataAPI.route('/processdata', methods=['POST'])
def process_data():
    print("processdata")

    data = request.get_json()

    msg = API().process_data(data, Database())

    print(msg)

    return json.dumps(msg)


from abc import abstractmethod

from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.ControlManager.ImageClassifierManager import ImageClassifierManager


class ImageClassifierAPIModule(APIModule):
    def __init__(self, manager: ImageClassifierManager):
        super(ImageClassifierAPIModule, self).__init__()
        self.manager = manager
        return

    @abstractmethod
    def get_function_mappings(self):
        raise NotImplementedError

    def get_classes(self, user, request, db):
        data = self.manager.get_classes(db)
        return Status(type=STATUS_OK, data={'classes':data})

    def save_class(self, user, request, db):
        data = self.manager.save_class(user, request['id'], request['data'], request['classname'], db)
        return Status(type=STATUS_OK, data=data)

    def start_training(self, user, request, db):
        data = self.manager.start_training(user, db)
        return Status(type=STATUS_OK, data=data)

    def get_performance_settings(self, user, request, db):
        data = self.manager.get_performance_settings(user, db)
        return Status(type=STATUS_OK, data=data)

    def set_performance_settings(self, user, request, db):
        data = self.manager.set_performance_settings(user, request['data'], db)
        return Status(type=STATUS_OK, data=data)

    def change_image_class(self, user, request, db):
        data = self.manager.change_image_class(user, request['ids'], request['newclass'], db)
        return Status(type=STATUS_OK, data=data)

    def get_image(self, user, request, db):
        data = self.manager.get_image(user, request['id'], db)
        print(data)
        return Status(type=STATUS_OK, data=data)

    def get_class_images(self, user, request, db):
        data = self.manager.get_class_images(user, request['class'], request['show'], request['offset'], db)
        return Status(type=STATUS_OK, data={'images':data})

    def upload_images(self, user, request, db):
        data = self.manager.upload_images(user, request['data'], request['class'], db)
        return Status(type=STATUS_OK, data=data)

    def classify_image(self, user, request, db):
        data = self.manager.classify_image(user, request['imagedata'], db)
        return Status(type=STATUS_OK, data=data)

    def delete_images(self, user, request, db):
        data = self.manager.delete_images(user, request['ids'], db)
        return Status(type=STATUS_OK, data=data)


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.RemoteControlManager import RemoteControlManager

ACTION_KEY_SET_REMOTE_DATA = "setremotedata"
ACTION_KEY_GET_REMOTE_DATA = "getremotedata"
ACTION_KEY_SET_REMOTE_CONTROL_ENABLED = "setremotecontrolenabled"
ACTION_KEY_CONNECT_REMOTE_ID_WITH_ACCOUNT = "connectremoteidwithaccount"

class RemoteControlAPIModule(APIModule):
    def __init__(self):
        super(RemoteControlAPIModule, self).__init__()
        self.remote_control_manager = RemoteControlManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_SET_REMOTE_DATA: self.set_remote_data,
            ACTION_KEY_GET_REMOTE_DATA: self.get_remote_data,
            ACTION_KEY_SET_REMOTE_CONTROL_ENABLED: self.set_remote_control_enabled,
            ACTION_KEY_CONNECT_REMOTE_ID_WITH_ACCOUNT: self.connect_remote_id_with_account
        }

        return mappings

    def set_remote_data(self, user, request, db):
        try:
            self.remote_control_manager.save_remote_data(user, request['remote_id'], request['linked_account'], db)
            return Status(type=STATUS_OK)
        except:
            return Status(type=STATUS_ERROR)

    def get_remote_data(self, user, request, db):
        try:
            data = self.remote_control_manager.load_remote_data(user, db)
            return Status(type=STATUS_OK, data=data)
        except:
            return Status(type=STATUS_ERROR)

    def set_remote_control_enabled(self, user, request, db):
        try:
            self.remote_control_manager.set_remote_control_enabled(user, request['enabled'], db)
            return Status(type=STATUS_OK)
        except:
            return Status(type=STATUS_ERROR)

    def connect_remote_id_with_account(self, user, request, db):
        try:
            success = self.remote_control_manager.connect_remote_id_with_account(user,
                                    request['accountname'], request['accountsecret'], db)
            if success:
                return Status(type=STATUS_OK)
            else:
                return Status(type=STATUS_ERROR)
        except:
            return Status(type=STATUS_ERROR)


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.RoomDataManager import RoomDataManager

ACTION_KEY_GET_ROOMDATA = "getroomdata"

class RoomDataAPIModule(APIModule):
    def __init__(self):
        super(RoomDataAPIModule, self).__init__()
        self.room_data_manager = RoomDataManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_ROOMDATA: self.get_room_data,
        }
        return mappings

    def get_room_data(self, user, request, db) -> Status:
        rooms = self.room_data_manager.get_room_data(user, request['room'], db)
        return Status(type=STATUS_OK, data={'roomdata':rooms})


#!/usr/bin/python
# -*- coding: utf-8 -*-
import zlib

class CompressionHandler:
    def __init__(self):
        return

    def compress_string(self, data: str) -> str:
        """
        Compresses the given data string
        :param data: the data string
        :return: the compresses string
        """

        data = data.encode('utf-8')

        #deflate_compress = zlib.compressobj(9, zlib.DEFLATED, -zlib.MAX_WBITS)
        zlib_compress = zlib.compressobj(9, zlib.DEFLATED, zlib.MAX_WBITS)
        #gzip_compress = zlib.compressobj(9, zlib.DEFLATED, zlib.MAX_WBITS | 16)

        #deflate_data = deflate_compress.compress(data) + deflate_compress.flush()
        zlib_data = zlib_compress.compress(data) + zlib_compress.flush()
        #gzip_data = gzip_compress.compress(data) + gzip_compress.flush()

        return zlib_data

    def decompress_string(self, data: str) -> str:
        """
        Decompresses the given compressed data string
        :param data: the compressed data string
        :return: the decompressed string
        """
        return zlib.decompress(data).decode('utf-8')

if __name__ == "__main__":
    compress_handler = CompressionHandler()

    plain_string = "hallo, mein name ist sascha."

    print("####PLAIN####")
    print(plain_string)

    compressed_string = compress_handler.compress_string(plain_string)

    print("####COMPRESSED####")
    print(compressed_string)

    decompressed_string = compress_handler.decompress_string(compressed_string)

    print("####DECOMPRESSED####")
    print(decompressed_string)
    print(plain_string)


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.ControlManager.EnergyDataManager import EnergyDataManager

ACTION_KEY_GET_ENERGY_DATA = "getenergydata"
ACTION_KEY_GET_DEVICE_ENERGY_DATA = "getdeviceenergydata"
ACTION_KEY_GET_ENERGY_COURSE = "getenergycourse"
ACTION_KEY_SET_POWER_COST = "setpowercost"
ACTION_KEY_GET_POWER_COST = "getpowercost"

class EnergyDataAPIModule(APIModule):
    def __init__(self):
        super(EnergyDataAPIModule, self).__init__()
        self.energy_manager = EnergyDataManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_ENERGY_DATA: self.get_energy_data,
            ACTION_KEY_GET_DEVICE_ENERGY_DATA: self.get_device_energy_data,
            ACTION_KEY_GET_ENERGY_COURSE: self.get_energy_course,
            ACTION_KEY_GET_POWER_COST: self.get_power_cost,
            ACTION_KEY_SET_POWER_COST: self.set_power_cost
        }

        return mappings

    def get_energy_data(self, user, request, db):
        data = self.energy_manager.get_energy_data(user, request['room'], request['devicetype'],
                                                           request['deviceid'],
                                                           request['von'], request['bis'], db)
        return Status(type=STATUS_OK, data={'energydata': data})

    def get_device_energy_data(self, user, request, db):
        data = self.energy_manager.get_device_energy_data(user, request['room'], request['devicetype'],
                                                                  request['deviceid'],
                                                                  request['von'], request['bis'], db)
        return Status(type=STATUS_OK, data=data)

    def get_energy_course(self, user, request, db):
        data = self.energy_manager.get_energy_course(user, request['room'], request['von'], request['bis'], db)
        return Status(type=STATUS_OK, data=data)

    def get_power_cost(self, user, request, db):
        data = self.energy_manager.set_power_cost(user, request['cost'], db)
        return Status(type=STATUS_OK, data={'powercost': data})

    def set_power_cost(self, user, request, db):
        self.energy_manager.get_power_cost(user, db)
        return Status(type=STATUS_OK)


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.ControlManager.BlindsManager import BlindsManager

ACTION_KEY_SET_BLINDS = "setblinds"
ACTION_KEY_SET_ROOM_BLINDS = "setroomblinds"
ACTION_KEY_GET_ALL_BLINDS = "getallblinds"

class BlindsAPIModule(APIModule):
    def __init__(self):
        super(BlindsAPIModule, self).__init__()
        self.blinds_manager = BlindsManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_SET_BLINDS: self.set_blinds,
            ACTION_KEY_SET_ROOM_BLINDS: self.set_room_blinds,
            ACTION_KEY_GET_ALL_BLINDS: self.get_all_blinds
        }

        return mappings

    def set_blinds(self, user, request, db):
        self.blinds_manager.set_blinds(user, request['type'], request['id'], request['value'], db)
        return Status(type=STATUS_OK)

    def set_room_blinds(self, user, request, db):
        self.blinds_manager.set_room_blinds(user, request['location'], request['value'], db)
        return Status(type=STATUS_OK)

    def get_all_blinds(self, user, request, db):
        data = self.blinds_manager.get_all_blinds(user, db)
        return Status(type=STATUS_OK, data={'blinds': data})


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.DeviceManager import DeviceManager

ACTION_KEY_ADD_EDIT_DEVICE = "addeditdevice"
ACTION_KEY_DELETE_DEVICE = "deletedevice"
ACTION_KEY_GET_DEVICE_DATA = "getdevicedata"

class DeviceAPIModule(APIModule):
    def __init__(self):
        super(DeviceAPIModule, self).__init__()
        self.device_manager = DeviceManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_ADD_EDIT_DEVICE: self.add_edit_device,
            ACTION_KEY_DELETE_DEVICE: self.delete_device,
            ACTION_KEY_GET_DEVICE_DATA: self.get_device_data
        }

        return mappings

    def add_edit_device(self, user, request, db):
        data = self.device_manager.add_edit_device(user, request['id'], request['type'], request['data'], db)
        return Status(type=STATUS_OK, data=data)

    def delete_device(self, user, request, db):
        data = self.device_manager.delete_device(user, request['type'], request['id'], db)
        return Status(type=STATUS_OK, data=data)

    def get_device_data(self, user, request, db):
        data = self.device_manager.get_device_data(user, request['type'], request['id'], db)
        return Status(type=STATUS_OK, data=data)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import urllib.error
import urllib.parse
import urllib.request

from Homevee.Exception import NoSuchTypeException


class KodiAPI:
    def __init__(self):
        return

    def media_remote_action(self, data, action):
        if action == "toggleplay":
            player = self.get_active_player(data)

            if player is None:
                return "notplaying"

            command = {
                'jsonrpc': '2.0', 'method': 'Player.PlayPause', 'params': {'playerid': int(player['playerid'])}, 'id': int(1)
            }

            result = self.send_kodi_command(command, data)

            result = json.loads(result)

            #print result

            if result['result']['speed'] is 0:
                return "play"
            elif result['result']['speed'] is 1:
                return "pause"
            raise Exception
        elif action == "up":
            method = "Input.Up"
        elif action == "down":
            method = "Input.Down"
        elif action == "left":
            method = "Input.Left"
        elif action == "right":
            method = "Input.Right"
        elif action == "select":
            method = "Input.select"
        elif action == "home":
            method = "Input.Home"
        elif action == "menu":
            method = "Input.ContextMenu"
        elif action == "volumedown":
            method = ""
        elif action == "volumeup":
            method = ""
        elif action == "togglemute":
            method = "Application.SetMute"
        else:
            raise NoSuchTypeException

        command = {
            'jsonrpc': '2.0', 'method': method, 'id': int(1)
        }

        return self.send_kodi_command(command, data)


    def get_movies(self, data, limit, offset):
        command = {
            'jsonrpc': '2.0', 'method': 'VideoLibrary.GetMovies', 'params':{
                'properties':['thumbnail', 'genre', 'year'], 'limits':{'start': int(offset), 'end': int(offset)+int(limit)},
                'sort':{'order': 'ascending', 'method': 'label'}},'id': 'libMovies'
        }

        return self.send_kodi_command(command, data)

    def get_tv_shows(self, data, limit, offset):
        command = {
            'jsonrpc': '2.0', 'method': 'VideoLibrary.GetTVShows', 'params':{
                'properties':['thumbnail', 'genre', 'year'], 'limits':{'start': int(offset), 'end': int(offset)+int(limit)},
                'sort':{'order': 'ascending', 'method': 'label'}},'id': 'libTvShows'
        }

        return self.send_kodi_command(command, data)

    def get_music(self, data, limit, offset):
        command = {
            'jsonrpc': '2.0', 'method': 'AudioLibrary.GetSongs', 'params':{
                'properties':['artist', 'album', 'track'], 'limits':{'start': int(offset), 'end': int(offset)+int(limit)},
                'sort':{'order': 'ascending', 'method': 'label'}},'id': 'libSongs'
        }

        return self.send_kodi_command(command, data)

    def get_artists(self, data, limit, offset):
        command = {
            'jsonrpc': '2.0', 'method': 'AudioLibrary.GetArtists', 'params':{
                'properties':['thumgnail'], 'limits':{'start': int(offset), 'end': int(offset)+int(limit)},
                'sort':{'order': 'ascending', 'method': 'label'}},'id': 'libArtists'
        }

        return self.send_kodi_command(command, data)

    def get_music_genres(self, data, limit, offset):
        command = {
            'jsonrpc': '2.0', 'method': 'AudioLibrary.GetGenres', 'params':{
                'properties':['thubmnail', 'title'], 'limits':{'start': int(offset), 'end': int(offset)+int(limit)},
                'sort':{'order': 'ascending', 'method': 'label'}},'id': 'libGenres'
        }

        return self.send_kodi_command(command, data)

    def get_albums(self, data, limit, offset):
        command = {
            'jsonrpc': '2.0', 'method': 'AudioLibrary.GetAlbums', 'params':{
                'properties':['artist', 'title', 'thumbnail'], 'limits':{'start': int(offset), 'end': int(offset)+int(limit)},
                'sort':{'order': 'ascending', 'method': 'label'}},'id': 'libAlbums'
        }

        return self.send_kodi_command(command, data)

    def get_movie_genres(self, data, limit, offset):
        command = {
            'jsonrpc': '2.0', 'method': 'VideoLibrary.GetGenres', 'params':{'type':'movie',
                'properties':['thumbnail', 'title'], 'limits':{'start': int(offset), 'end': int(offset)+int(limit)},
                'sort':{'order': 'ascending', 'method': 'label'}},'id': 'libGenres'
        }

        return self.send_kodi_command(command, data)

    def get_tv_show_seasons(self, data, show_id, limit, offset):
        command = {
            'jsonrpc': '2.0', 'method': 'VideoLibrary.GetSeasons', 'params':{
                'properties':['thumbnail'], 'limits':{'start': int(offset), 'end': int(offset)+int(limit)},
                'sort':{'order': 'ascending', 'method': 'label', 'ignorearticel': True}},'id': 'libSeasons', 'tvshowid': show_id
        }

        return self.send_kodi_command(command, data)

    def send_text(self, data, text):
        command = {
            'jsonrpc': '2.0', 'method': 'Input.SendText', 'params': {'text': text, 'done': True}, 'id': 1
        }

        return self.send_kodi_command(command, data)

    def get_active_player(self, data):
        command = {
            'jsonrpc': '2.0', 'method': 'Player.GetActivePlayers', 'id': 1
        }

        result = self.send_kodi_command(command, data)

        result = json.loads(result)

        #print result

        for player in result['result']:
            #print player
            return player

        return None

    #def remote_action(data, action):

    def send_kodi_command(self, command, data):
        command = json.dumps(command)

        ip = data['IP']
        port = data['PORT']

        if port is not "" and port is not None:
            port = ":" + port
        else:
            port = ""

        link = "http://"+ip+port+"/jsonrpc?request="+urllib.parse.quote(command)

        try:
            result = urllib.request.urlopen(link).read()
        except:
            result = None

        return result


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import json
import traceback

from Homevee.Helper import Logger
from Homevee.Item.Room import Room
from Homevee.Manager.CalendarManager import CalendarManager
from Homevee.Manager.ControlManager.ThermostatManager import ThermostatManager
from Homevee.Manager.NutritionDataManager import NutritionDataManager
from Homevee.Manager.SensorDataManager import SensorDataManager
from Homevee.Manager.WeatherManager import WeatherManager
from Homevee.Utils.Database import Database
from Homevee.Utils.DeviceTypes import *

STATIC_DASHBOARD_ITEMS = ['calendar', 'weather', 'tvprogramme', 'drivingtime', 'nutritionmanager']

class DashboardManager():
    def __init__(self):
        return

    def get_user_dashboard(self, user, db: Database = None):
        if db is None:
            db = Database()
        user_dashboard = user.dashboard_data

        if user_dashboard is None:
            data = {'static': [], 'dynamic': []}
        else:
            data = json.loads(user_dashboard)

        dashboard_data = {}
        dashboard_data['static'] = data['static']

        if dashboard_data is not None and dashboard_data != "":

            dashboard_keys = []
            for item in dashboard_data['static']:
                Logger.log(item)
                dashboard_keys.append(item['id'])


            for item in STATIC_DASHBOARD_ITEMS:
                if item not in dashboard_keys:
                    Logger.log(item+" not existing")
                    dashboard_data['static'].append({'id': item, 'checked': False})

            dashboard_data['dynamic'] = []

            devices_map = {}
            for item in data['dynamic']:
                if item is None:
                    continue

                found = False

                if (item['type'] not in devices_map):
                    devices_map[item['type']] = []

                if item['id'] not in devices_map[item['type']]:
                    devices_map[item['type']].append(item['id'])
                else:
                    found = True

                if not found:
                    dashboard_data['dynamic'].append(self.get_device_info(user, item['type'], item['id']))

            return dashboard_data
        else:
            return None

    def get_user_dashboard_items(self, user, db: Database = None):
        if db is None:
            db = Database()
        dashboard_array = []

        data = user.dashboard_data

        if data is None:
            return dashboard_array

        data = json.loads(data)

        static_items = data['static']

        static_data = {}

        for item in static_items:
            static_data[item['id']] = item['checked']

        dynamic_temp_items = data['dynamic']
        devices_map = {}
        dynamic_items = []

        Logger.log(dynamic_temp_items)

        for item in dynamic_temp_items:
            if item is None:
                continue

            found = False

            if (item['type'] not in devices_map):
                devices_map[item['type']] = []

            if item['id'] not in devices_map[item['type']]:
                devices_map[item['type']].append(item['id'])
            else:
                found = True

            if not found:
                dynamic_items.append(item)

        Logger.log(dynamic_items)

        # Wetter hinzufügen
        if 'weather' in static_data:
            if static_data['weather']:
                weather_data = WeatherManager().get_weather(1)
                dashboard_array.append({'id': 'weather', 'data': weather_data[0]})

        # TV Programm hinzufügen
        if 'tvprogramme' in static_data:
            if static_data['tvprogramme']:
                dashboard_array.append({'id': 'tvprogramme', 'data': None})

        # Kalender hinzufügen
        if 'calendar' in static_data:
            if static_data['calendar']:
                calendar_entries = CalendarManager().get_calendar_day_items(user, datetime.datetime.now().strftime("%Y-%m-%d"))

                items = calendar_entries

                data = None
                for item in items:
                    start = datetime.datetime.strptime(item.start, '%H:%M')
                    dnow = datetime.datetime.now()

                    if (dnow.time() > start.time()):
                        continue

                    if data is None:
                        data = item.name + " um " + item.start
                    else:
                        data += '\n' + item.name + " um " + item.start + " Uhr"

                if data is None:
                    data = 'Keine Termine heute'

                dashboard_array.append({'id': 'calendar', 'data': data})

        # Fahrzeit
        if 'drivingtime' in static_data:
            if static_data['drivingtime']:
                dashboard_array.append({'id': 'drivingtime', 'data': '27 min'})

        # Ernährungsmanager
        try:
            if 'nutritionmanager' in static_data:
                if static_data['nutritionmanager']:
                    data_item = NutritionDataManager().get_user_nutrition_overview(user)

                    if 'nutrition_day_data' in data_item:
                        calories_left = data_item['nutrition_day_data']['calories_left']
                        if (calories_left >= 0):
                            text = str(calories_left) + ' Kalorien übrig'
                        else:
                            text = str(calories_left) + ' Kalorien über deinem Ziel'

                        dashboard_array.append({'id': 'nutritionmanager', 'data': text})
                    else:
                        dashboard_array.append({'id': 'nutritionmanager',
                                                'data': "Ernährungsmanager noch nicht eingerichtet"})
        except:
            if Logger.IS_DEBUG:
                traceback.print_exc()
            pass

        if len(dynamic_items) > 0:
            dashboard_array.append({'id': 'favouritedevices', 'data': None})

        return dashboard_array

    def edit_user_dashboard(self, user, dashboard_data, db: Database = None):
        if db is None:
            db = Database()

        try:
            dashboard_data = json.loads(dashboard_data)

            #Check duplicates

            dynamic_temp_items = dashboard_data['dynamic']
            devices_map = {}
            dynamic_items = []

            #Logger.log(dynamic_temp_items)

            for item in dynamic_temp_items:
                found = False

                if (item['type'] not in devices_map):
                    devices_map[item['type']] = []

                if item['id'] not in devices_map[item['type']]:
                    devices_map[item['type']].append(item['id'])
                else:
                    found = True

                if not found:
                    dynamic_items.append(item)

            #Logger.log(dynamic_items)

            dashboard_data['dynamic'] = dynamic_items

            #Check duplicates end

            dashboard_data_json = json.dumps(dashboard_data)

            print(dashboard_data_json)

            db.update("UPDATE USERDATA SET DASHBOARD_DATA = :data WHERE USERNAME == :username",
                        {'data': dashboard_data_json, 'username': user.username})

            return True
        except:
            if Logger.IS_DEBUG:
                traceback.print_exc()
            return False

    def get_user_favourite_devices(self, user, db: Database = None):
        if db is None:
            db = Database()
        data = json.loads(user.dashboard_data)

        dynamic_items = data['dynamic']

        #Logger.log("data: "+str(data))

        fav_array = self.get_dynamic_items(user, dynamic_items)

        return fav_array

    def get_dynamic_items(self, user, dynamic_items, db: Database = None):
        if db is None:
            db = Database()
        rooms = Room.load_all(db)

        favourites = {}

        room_map = {}

        for room in rooms:
            if(user.has_permission(room.id)):
                favourites[int(room.id)] = []
                room_map[room.id] = room.get_dict()
            else:
                continue

        types = {}

        for item in dynamic_items:
            #print item
            if item['type'] not in types:
                types[item['type']] = []

            types[item['type']].append(item['id'])
            #Logger.log(types)

        for type in types:
            ids = types[type]
            if len(ids) > 0:
                for id in ids:
                    item = self.get_device_info(user, type, id)

                    if item is None:
                        continue

                    #Logger.log(favourites)

                    favourites[int(item['room'])].append(item)

        output = []
        for location in list(favourites.keys()):
            room_item = room_map[location]

            if len(favourites[location]) > 0:
                room_item['devices'] = favourites[location]
                output.append(room_item)

        return output

    def get_device_info(self, user, type, id, db: Database = None):
        if db is None:
            db = Database()
        Logger.log(type + " - " + id)

        device_data = None

        try:
            if type == FUNKSTECKDOSE:
                item = db.select_one("SELECT * FROM FUNKSTECKDOSEN WHERE DEVICE == :id", {'id': id})
                device_data = {'room': item['ROOM'], 'name': item['NAME'], 'icon': item['ICON'],
                               'value': item['ZUSTAND'] == "1",
                               'type': 'switch', 'devicetype': type, 'id': id}
            elif type == PHILIPS_HUE_LIGHT:
                item = db.select_one("SELECT * FROM PHILIPS_HUE_LIGHTS WHERE ID == :id", {'id': id})
                device_data = {'room': item['LOCATION'], 'name': item['NAME'],
                               'icon': item['ICON'],
                               'value': {'is_on': False, 'brightness': 100, 'color': item['HUE']},
                               'type': 'rgb', 'devicetype': type, 'id': id}
            #URL-Geräte
            elif type == URL_RGB_LIGHT:
                item = db.select_one("SELECT * FROM URL_RGB_LIGHT WHERE ID = :id", {'id': id})
                device_data = {'name': item['NAME'], 'room': item['LOCATION'], 'id': item['ID'], 'type': 'rgb',
                               'icon': item['ICON'], 'value': {'is_on': False, 'brightness': 100, 'color': item['LAST_COLOR']}}
            elif type == URL_TOGGLE:
                item = db.select_one("SELECT * FROM URL_TOGGLE WHERE ID = :id",{'id': id})
                device_data = {'name': item['NAME'], 'room': item['LOCATION'], 'id': item['ID'],
                                    'icon': item['ICON'], 'value': '', 'type': 'toggle'}
            #Z-Wave
            elif type == ZWAVE_THERMOSTAT:
                item = db.select_one("SELECT * FROM ZWAVE_THERMOSTATS WHERE THERMOSTAT_ID == :id", {'id': id})
                data = ThermostatManager().get_thermostat_info(user, item['RAUM'], type, item['THERMOSTAT_ID'])
                device_data = {'name': item['NAME'], 'id': item['THERMOSTAT_ID'],
                        'icon': item['ICON'], 'room': item['RAUM'], 'type': 'heating',
                        'value': data['value'], 'min': data['min'], 'max': data['max'], 'step': 0.5}
            elif type == ZWAVE_DIMMER:
                item = db.select_one("SELECT * FROM ZWAVE_DIMMER WHERE ID == :id", {'id': id})
                device_data = {'name': item['NAME'], 'id': item['THERMOSTAT_ID'],
                        'icon': item['ICON'], 'room': item['LOCATION'], 'type': 'heating',
                        'value': item['VALUE']}
            elif type == ZWAVE_POWER_METER:
                item = db.select_one("SELECT * FROM ZWAVE_THERMOSTATS WHERE DEVICE_ID == :id", {'id': id})
                device_data = {'name': item['NAME'], 'id': item['DEVICE_ID'],
                        'icon': item['ICON'], 'room': item['LOCATION'], 'type': 'heating',
                        'value': item['VALUE']}
            elif type == ZWAVE_SENSOR:
                item = db.select_one("SELECT * FROM ZWAVE_SENSOREN WHERE ID == :id", {'id': id})
                device_data = {'name': item['SHORTFORM'], 'id': item['ID'],
                        'icon': item['ICON'], 'room': item['RAUM'], 'type': 'value',
                               'value': SensorDataManager().get_sensor_data(user, item['RAUM'], type, id, 1)}
            elif type == ZWAVE_SWITCH:
                item = db.select_one("SELECT * FROM ZWAVE_SWITCHES WHERE THERMOSTAT_ID == :id", {'id': id})
                device_data = {'name': item['NAME'], 'id': item['THERMOSTAT_ID'],
                        'icon': item['ICON'], 'room': item['LOCATION'], 'type': 'heating',
                        'value': item['VALUE']}
            #MAX Cube
            elif type == MAX_THERMOSTAT:
                item = db.select_one("SELECT * FROM MAX_THERMOSTATS WHERE ID == :id", {'id': id})
                data = ThermostatManager().get_thermostat_info(user, item['RAUM'], type, id)
                device_data = {'name': item['NAME'], 'id': item['ID'], 'room': item['RAUM'], 'type': 'heating',
                        'icon': item['ICON'],
                        'value': data['value'], 'min': data['min'], 'max': data['max'], 'step': 0.5}
            #Wake on Lan
            elif type == WAKE_ON_LAN:
                item = db.select_one("SELECT * FROM WAKE_ON_LAN WHERE DEVICE == :id", {'id': id})
                device_data = {'name': item['NAME'], 'id': item['DEVICE'], 'icon': item['ICON'], 'room': item['LOCATION'],
                               'value': '', 'type': 'wakeonlan'}
            elif type == XBOX_ONE_WOL:
                item = db.select_one("SELECT * FROM XBOX_ONE_WOL WHERE ID == :id", {'id': id})
                device_data = {'name': item['NAME'], 'id': item['ID'], 'icon': item['ICON'], 'room': item['LOCATION'],
                               'value': '', 'type': 'wakeonlan'}

            #MQTT Sensor
            elif type == MQTT_SENSOR:
                item = db.select_one("SELECT * FROM MQTT_SENSORS WHERE ID == :id", {'id': id})
                device_data = {'name': item['NAME'], 'id': item['ID'],
                               'icon': item['ICON'], 'room': item['ROOM'], 'type': 'value',
                               'value': SensorDataManager().get_sensor_data(user, item['ROOM'], type, id, 1)}
        except:
            Logger.log("Error loading device")

        if device_data is None or not user.has_permission(device_data['room']):
            return None

        device_data['roomname'] = Room.get_name_by_id(device_data['room'])
        device_data['devicetype'] = type

        return device_data

        '''//DIY
            case "DIY Gerät":
                $dbQuery = $db->prepare("SELECT * FROM 'DIY_DEVICES' WHERE DEVICE == :id");
                $dbQuery->execute($queryParams);
                
                
                $query = $dbQuery->fetch(PDO::FETCH_ASSOC);
                $devicedata = array("room" => $query['ROOM'], "name" => $query['NAME'], "icon" => $query['ICON'],
                "value" => "", "type" => "button");
                break;
            case "DIY Sensor":
                $dbQuery = $db->prepare("SELECT * FROM 'DIY_SENSORS' WHERE ID == :id");
                $dbQuery->execute($queryParams);
                
                
                $query = $dbQuery->fetch(PDO::FETCH_ASSOC);
                $devicedata = array("room" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON'],
                "value" => getSensorData($query['RAUM'], $type, $id, "1", $db), "type" => "value");
            case "DIY Fensterkontakt":
                $dbQuery = $db->prepare("SELECT * FROM 'DIY_REEDSENSORS' WHERE ID == :id");
                $dbQuery->execute($queryParams);
                
                
                $query = $dbQuery->fetch(PDO::FETCH_ASSOC);
                $devicedata = array("room" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON'],
                "value" => getReedData($query['RAUM'], $type, $id, $db), "type" => "reed");
                break;
            case "DIY Anwesenheitssensor":
                $dbQuery = $db->prepare("SELECT * FROM 'DIY_PRESENCESENSORS' WHERE ID == :id");
                $dbQuery->execute($queryParams);
                
                
                $query = $dbQuery->fetch(PDO::FETCH_ASSOC);
                $devicedata = array("room" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON'],
                "value" => getReedData($query['RAUM'], $type, $id, $db), "type" => "presence");
                break;
            case "DIY Schalter":
                $dbQuery = $db->prepare("SELECT * FROM 'DIY_SWITCHES' WHERE ID == :id");
                $dbQuery->execute($queryParams);
                
                
                $query = $dbQuery->fetch(PDO::FETCH_ASSOC);
                $devicedata = array("room" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON'],
                "value" => getModes($query['RAUM'], $type, $id, $db) === "1", "type" => "switch");
                break;
            case "IP Kamera":
                $dbQuery = $db->prepare("SELECT * FROM 'IP_CAMERA' WHERE ID == :id");
                $dbQuery->execute($queryParams);
                
                $query = $dbQuery->fetch(PDO::FETCH_ASSOC);
                
                $ip = $query['IP'];
                $port = $query['PORT'];
                    
                if($port != null && $port != ""){
                    $port = ":$port";
                }
                    
                $path = $query['PATH'];
                    
                $url = "http://$ip$port$path";
                
                $devicedata = array('room' => $query['LOCATION'], 'name' => $query['NAME'], 'icon' => $query['ICON'],
                'value' => "", 'url' => $url, 'username' => $query['USERNAME'], 'password' => $query['PASSWORD'],
                'type' => "ipcamera");
                break;
                
            default:
                return null;
        }
        
        if(!hasPermission($devicedata['room'], $db) || $devicedata['room'] == null){
            return null;
        }
        else{
            $devicedata['roomname'] = $roomArray[$devicedata['room']];
            $devicedata['devicetype'] = $type;
            $devicedata['id'] = $id;
            
            return $devicedata;
        }
    }
    ?>
        '''



from Homevee.AIAgents.ImageClassifierRetrainer import ImageClassifierRetrainer


class ARControlRetrainer(ImageClassifierRetrainer):
    def __init__(self):
        image_dir_name = "ar_control"
        model_dir_name = "ar_control_temp"
        retrainer_name = "AR-Control"
        super(ARControlRetrainer, self).__init__(image_dir_name, model_dir_name, retrainer_name)


#!/usr/bin/python
# -*- coding: utf-8 -*-

from Homevee.DeviceAPI import rademacher_homepilot
from Homevee.Exception import NoPermissionException, NoSuchTypeException
from Homevee.Item.Room import Room
from Homevee.Utils.Database import Database
from Homevee.Utils.DeviceTypes import *


class BlindsManager:
    def __init__(self):
        return

    def set_blinds(self, user, type, id, new_position, db, check_user=True):
        if type == RADEMACHER_BLIND_CONTROL:
            data = db.select_one("SELECT LOCATION FROM HOMEPILOT_BLIND_CONTROL WHERE ID = :id", {'id': id})
            if check_user and not user.has_permission(data['LOCATION']):
                raise NoPermissionException

            return rademacher_homepilot.blinds_control.control_blinds(id, new_position)
        else:
            raise NoSuchTypeException

    def set_room_blinds(self, user, room, new_position, db: Database = None):
        if db is None:
            db = Database()

            devices = self.get_blinds(user, room)

            for device in devices['blinds']:
                id = device['id']
                type = device['type']
                result = self.set_blinds(user, type, id, new_position, db)
                if 'result' not in result or result['result'] != 'ok':
                    raise Exception

    def get_all_blinds(self, user, db: Database = None):
        if db is None:
            db = Database()

        rooms = db.select_all("SELECT * FROM ROOMS")
        blinds = []
        for room in rooms:
            if not user.has_permission(room['LOCATION']):
                continue

            room_blinds = BlindsManager().get_blinds(user, room['LOCATION'])

            if len(room_blinds) == 0:
                continue

            room_blinds_item = {'name': room['NAME'], 'location': room['LOCATION'], 'icon': room['ICON'],
                                'blind_array': room_blinds}
            blinds.append(room_blinds_item)
        return blinds

    def get_blinds(self, user, location, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission(location):
            return {'result': 'nopermission'}

        if isinstance(location, Room):
            location = location.id

        blinds = []

        #Rademacher HomePilot
        items = db.select_all("SELECT * FROM HOMEPILOT_BLIND_CONTROL WHERE LOCATION = :location",
                    {'location': location})
        for item in items:
            value = int(item['LAST_POS'])

            if value is None:
                value = 0

            blinds.append({'name': item['NAME'], 'id': item['ID'], 'location': location,
                           'icon': item['ICON'], 'value': value, 'type': RADEMACHER_BLIND_CONTROL})

        #Andere Gerätetypen

        return blinds



#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import os

import requests

from Homevee.DeviceAPI import zwave, philips_hue, rademacher_homepilot, max_cube
from Homevee.Exception import NoPermissionException, NoSuchTypeException
from Homevee.Helper import Logger
from Homevee.Item.Gateway import *
from Homevee.Item.Status import *


class GatewayManager:
    def __init__(self):
        return

    def add_edit_gateway(self, user, type, username, password, change_pw, ip, port, gateway_type, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission("admin"):
            raise NoPermissionException

        param_array = {'name': type, 'ip': ip, 'port': port, 'key1': username, 'type': gateway_type}

        if change_pw == "true":
            param_array['key2'] = password

            db.update("UPDATE OR IGNORE 'GATEWAYS' SET IP = :ip, PORT = :port, KEY1 = :key1, KEY2 = :key2 TYPE = :type, WHERE NAME = :name;",
                        param_array)

            db.insert("INSERT OR IGNORE INTO 'GATEWAYS' (NAME, IP, PORT, KEY1, KEY2, TYPE) VALUES (:name, :ip, :port, :key1, :key2, :type);",
                        param_array)
        else:
            db.update("UPDATE OR IGNORE 'GATEWAYS' SET IP = :ip, PORT = :port, KEY1 = :key1 WHERE NAME = :name;",
                        param_array)

            db.insert("INSERT OR IGNORE INTO 'GATEWAYS' (NAME, IP, PORT, KEY1, TYPE) VALUES (:name, :ip, :port, :key1, :type);",
                        param_array)

    def delete_gateway(self, user, key, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission("admin"):
            raise NoPermissionException

        gateway = Item.load_from_db(Gateway, key)
        gateway.delete(db)

    def get_gateways(self, user, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission("admin"):
            raise NoPermissionException

        results = db.select_all("SELECT * FROM GATEWAYS", {})

        gateways = []

        for gateway in results:
            gateways.append({'name': gateway['NAME'], 'ip': gateway['IP'], 'port': gateway['PORT'],
                                'key1': gateway['KEY1'], 'type': gateway['TYPE']})

        return {'gateways': gateways, 'gatewaytypesleft': self.get_gateway_types(user, db)}

    def get_gateway_types(self, user, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission("admin"):
            raise NoPermissionException

        gateway_types = [
            {'key': Z_WAVE_GATEWAY, 'type': 'user'},
            {'key': FUNKSTECKDOSEN_CONTROLLER, 'type': 'url'},
            {'key': MAX_CUBE, 'type': 'url'},
            {'key': MQTT_BROKER, 'type': 'user'},
            {'key': PHILIPS_HUE_BRIDGE, 'type': 'apikey'},
            {'key': MIYO_CUBE, 'type': 'apikey'},
            {'key': RADEMACHER_HOMEPILOT, 'type': 'url'},
        ]

        results = db.select_all("SELECT NAME FROM GATEWAYS")

        for gateway in results:
            name = gateway['NAME']
            for i in range(0, len(gateway_types)-1):
                if gateway_types[i]['key'] == name:
                    del gateway_types[i]

        return gateway_types

    def connect_gateway(self, user, type, ip, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission("admin"):
            raise NoPermissionException

        if type == PHILIPS_HUE_BRIDGE:
            while True:
                #print("try connecting...")

                data = {"devicetype": "Homevee#system"}

                response = requests.post("http://" + ip + "/api", data=json.dumps(data))

                result = response.text

                #print("result: "+result)

                result = json.loads(result)

                result = result[0]

                if "error" in result:
                    if result['error']['type'] == 101:
                        return {'result': 'error', 'msg': 'Drücke bitte die Taste auf deinem Philips Hue Gateway.'}
                elif "success" in result:
                    user = result['success']['username']

                    self.add_edit_gateway(user.username, type, user, "", "", ip, "80", "apikey", db)

                    return Status(type=STATUS_OK).get_dict()
        elif type == MIYO_CUBE:
            while True:
                #print("try connecting...")

                response = requests.get("http://" + ip + "/api/link")

                result = response.text

                #print("result: "+result)

                result = json.loads(result)

                if "error" in result:
                    if result['errorLoc'] == "NOTIFY_ERROR_LINKINACTIVE":
                        return {'result': 'error', 'msg': 'Drücke bitte die Taste hinten auf deinem MIYO Cube.'}
                elif "success" in result:
                    user = result['success']['username']

                    self.add_edit_gateway(user.username, type, user, "", "", ip, "80", "apikey", db)

                    return Status(type=STATUS_OK).get_dict()
        else:
            return Status(type=STATUS_ERROR).get_dict()

    def get_gateway_devices(self, user, type, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission("admin"):
            raise NoPermissionException

        if type == Z_WAVE_GATEWAY:
            return zwave.get_devices.get_devices(db)
        if type == PHILIPS_HUE_BRIDGE:
            return philips_hue.get_devices(db)
        if type == RADEMACHER_HOMEPILOT:
            return rademacher_homepilot.get_devices(db)
        elif type == MAX_CUBE:
            gateway = Item.load_from_db(Gateway, MAX_CUBE, db)
            Logger.log(gateway.ip)
            devices = max_cube.get_devices(gateway.ip)
            if devices is not None:
                return devices
            else:
                return []
        elif type == "Network":
            network_devices = []
            hostnames = os.popen("arp -a").read()
            ip_adresses = os.popen("arp -a").read()
            Logger.log(hostnames)
            # print ip_adresses
            # for i in range(0, len(hostnames)):

            return network_devices
        else:
            raise NoSuchTypeException


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Item import Item
from Homevee.Item.CalendarEntry import CalendarEntry
from Homevee.Utils.Database import Database

class CalendarManager():
    def __init__(self):
        return

    def get_calendar_item_dates(self, user, year, db: Database = None):
        if db is None:
            db = Database()
        return CalendarEntry.get_calendar_item_dates(year)

    def get_calendar_day_items(self, user, date, db: Database = None):
        if db is None:
            db = Database()
        calendar_items = CalendarEntry.load_all_by_date(date)
        return calendar_items

    def delete_entry(self, user, id, db: Database = None):
        if db is None:
            db = Database()
        calendar_entry = Item.load_from_db(CalendarEntry, id)
        return calendar_entry.delete(db)

    def add_edit_entry(self, user, entry_id, name, date, start, end, note, address, db: Database = None):
        if db is None:
            db = Database()
        try:
            calendar_entry = Item.load_from_db(CalendarEntry, entry_id)
            calendar_entry.name = name
            calendar_entry.date = date
            calendar_entry.start = start
            calendar_entry.end = end
            calendar_entry.note = note
            calendar_entry.address = address
        except:
            calendar_entry = CalendarEntry(name, date, start, end, note, address)

        return calendar_entry.save(db)


from Homevee.Exception import DatabaseSaveFailedException, InvalidParametersException
from Homevee.Item import Item
from Homevee.Utils.Database import Database

class APIKey(Item):
    def __init__(self, name, key, id=None):
        super(APIKey, self).__init__()
        self.id = id
        self.key = key
        self.name = name

    def delete(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            db.delete("DELETE FROM API_KEYS WHERE ID == :id", {'id': self.id})
            return True
        except:
            return False

    def save(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            if (self.id is None or self.id == ""):
                new_id = db.insert("""INSERT INTO API_KEYS (SERVICE_NAME, API_KEY) VALUES (:name, :key)""",
                            {'name': self.name, 'key': self.key})
                self.id = new_id

            # update
            else:
                db.update("""UPDATE API_KEYS SET SERVICE_NAME = :name, API_KEY = :key WHERE ID = :id""",
                                {'name': self.name, 'key': self.key, 'id': self.id})
                #TODO add generated id to object
        except:
            raise DatabaseSaveFailedException("Could not save event to database")

    def build_dict(self):
        dict = {
            'id': self.id,
            'name': self.name,
            'key': self.key,
        }
        return dict

    @staticmethod
    def load_all_from_db(query, params, db: Database = None):
        if db is None:
            db = Database()
        items = []
        for result in db.select_all(query, params):
            item = APIKey(result['SERVICE_NAME'], result['API_KEY'], result['ID'])
            items.append(item)
        return items

    @staticmethod
    def load_all_ids_from_db(ids, db: Database = None):
        if db is None:
            db = Database()
        return APIKey.load_all_from_db('SELECT * FROM API_KEYS WHERE ID IN (%s)'
                                              % ','.join('?' * len(ids)), ids, db)

    @staticmethod
    def find_by_name(name, db: Database = None):
        if db is None:
            db = Database()
        results = APIKey.load_all_from_db('SELECT * FROM API_KEYS WHERE SERVICE_NAME = :name', {'name': name}, db)

        if len(results) > 0:
            return results[0]
        else:
            return None

    @staticmethod
    def load_all(db: Database):
        return APIKey.load_all_from_db('SELECT * FROM API_KEYS', {}, db)

    @staticmethod
    def create_from_dict(dict):
        try:
            id = dict['id']
            name = dict['name']
            key = dict['key']

            item = APIKey(name, key, id)

            return item
        except:
            raise InvalidParametersException("Person.create_from_dict(): invalid dict")


from Homevee.APIModule import APIModule
from Homevee.Item.Event import Event
from Homevee.Item.Status import *
from Homevee.Manager.EventManager import EventManager

ACTION_KEY_GET_EVENTS = "getevents"
ACTION_KEY_GET_EVENT_TYPES = "geteventtypes"
ACTION_KEY_GET_UNSEEN_EVENTS = "getunseenevents"

class EventAPIModule(APIModule):
    def __init__(self):
        super(EventAPIModule, self).__init__()
        self.event_manager = EventManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_EVENTS: self.get_events,
            ACTION_KEY_GET_EVENT_TYPES: self.get_event_types,
            ACTION_KEY_GET_UNSEEN_EVENTS: self.get_unseen_events
        }
        return mappings

    def get_events(self, user, request, db):
        events = self.event_manager.get_events(user, type, request['limit'], request['offset'], db)
        return Status(type=STATUS_OK, data={'events': Event.list_to_dict(events)})

    def get_event_types(self, user, request, db):
        types = self.event_manager.get_event_types(db)
        return Status(type=STATUS_OK, data={'types': types})

    def get_unseen_events(self, user, request, db):
        count = self.event_manager.get_unseen_events(user, db)
        return Status(type=STATUS_OK, data={'count': count})


import base64
import io
import json
import os
import time
import traceback
from _thread import start_new_thread

from PIL import Image

from Homevee.Exception import NoPermissionException, AlreadyTrainingException
from Homevee.Helper import Logger
from Homevee.Utils import Constants, FileUtils
from Homevee.Utils.Database import Database


class ImageClassifierManager:
    def __init__(self, data_table, learning_data_table, settings_prefix, is_training_tag, classifier_tag, classifier, retrainer):
        self.data_table = data_table
        self.learning_data_table = learning_data_table
        self.settings_prefix = settings_prefix
        self.is_training_tag = is_training_tag
        self.classifier_tag = classifier_tag
        self.classifier = classifier
        self.retrainer = retrainer
        return

    def get_classes(self, db: Database = None):
        results = db.select_all("SELECT * FROM "+self.data_table)
        classes = []
        for row in results:
            item = {'id': int(row['ID']), 'name': row['NAME'], }
            classes.append(item)
        item = {'id': None, 'name': 'Nicht zugeordnet'}
        classes.append(item)
        return classes

    def save_class(self, user, id, data, class_name, db: Database = None):
        if db is None:
            db = Database()
        return

    def get_class_data(self, id, db: Database = None):
        if db is None:
            db = Database()
        return db.select_one("SELECT * FROM "+self.data_table+" WHERE ID = :id;", {'id': id})

    def start_training(self, user, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission('admin'):
            raise NoPermissionException
        is_training = db.get_server_data(self.is_training_tag)
        if (is_training is not None and is_training == "true"):
            raise AlreadyTrainingException
        start_new_thread(self.training_thread, (user, None))

    def get_performance_settings(self, user, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission('admin'):
            raise NoPermissionException
        tf_data = {}
        indices = ['TF_TRAINING_STEPS', 'TF_MODEL_SIZE', 'TF_MODEL_IMAGE_SIZE']
        for index in indices:
            result = db.select_one("SELECT VALUE FROM SERVER_DATA WHERE KEY = :key",
                                   {'key': self.settings_prefix+index})
            if (result is not None and 'VALUE' in result):
                tf_data[index] = result['VALUE']
        return {'data': tf_data, 'graph_sizes': self.classifier.get_graph_sizes(),
                'image_sizes': self.retrainer.get_image_sizes()}

    def set_performance_settings(self, user, data, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission('admin'):
            raise NoPermissionException
        data = json.loads(data)
        indices = ['TF_TRAINING_STEPS', 'TF_MODEL_SIZE', 'TF_MODEL_IMAGE_SIZE']
        for index in indices:
            param_array = {'value': data[index], 'key': self.settings_prefix+index}
            db.update("UPDATE OR IGNORE SERVER_DATA SET VALUE = :value WHERE KEY = :key;", param_array)
            db.insert("INSERT OR IGNORE INTO SERVER_DATA (VALUE, KEY) VALUES (:value, :key);", param_array)

    def training_thread(self, user, d):
        db = Database()
        db.set_server_data(self.is_training_tag, False)
        try:
            self.retrainer.start_training(user.username)
        except:
            if (Logger.IS_DEBUG):
                traceback.print_exc()
        db.set_server_data(self.is_training_tag, "false")

    def delete_images(self, user, ids, db: Database = None):
        if db is None:
            db = Database()
        ids = json.loads(ids)
        for id in ids:
            Logger.log("deleting: " + str(id))
            item = db.select_one("SELECT * FROM "+self.learning_data_table+" WHERE ID == :id;", {'id': id})
            if (item is not None):
                rel_path = item['PATH']
                image_path = os.path.join(Constants.DATA_DIR, rel_path)
                if os.path.exists(image_path):
                    os.remove(image_path)
                else:
                    Logger.log(image_path + " not found")
                db.delete("DELETE FROM "+self.learning_data_table+" WHERE ID == :id;", {'id': id})

    def change_image_class(self, user, ids, new_class, db: Database = None):
        if db is None:
            db = Database()
        if new_class == "-1":
            new_class = None
        ids = json.loads(ids)
        Logger.log(ids)
        for id in ids:
            db.update("UPDATE "+self.learning_data_table+" SET CLASS_ID = :key WHERE ID == :id;",
                      {'key': new_class, 'id': id})

    def get_image(self, user, id, db: Database = None):
        if db is None:
            db = Database()
        result = db.select_one("SELECT PATH FROM " + self.learning_data_table + " WHERE ID == :id", {'id': id})
        rel_path = result['PATH']
        path = os.path.join(Constants.DATA_DIR, rel_path)
        Logger.log(("Path: " + path))
        try:
            im = Image.open(path)
        except:
            return None
        size = 50
        image_dimensions = size, size
        im.thumbnail(image_dimensions, Image.ANTIALIAS)
        buffer = io.BytesIO()
        im.save(buffer, format="JPEG")
        encoded_string = base64.b64encode(buffer.getvalue())
        encoded_string = encoded_string.decode('utf-8')
        im.close()
        return encoded_string

    def get_images(self, user, class_name, show, offset, db: Database = None):
        if db is None:
            db = Database()
        param_array = {'limit': show, 'offset': offset}

        if class_name is None or class_name == "-1":
            results = db.select_all(
                "SELECT * FROM "+self.learning_data_table+" WHERE CLASS_ID IS NULL LIMIT :limit OFFSET :offset",
                param_array)
        else:
            param_array['key'] = class_name
            results = db.select_all(
                "SELECT * FROM "+self.learning_data_table+" WHERE CLASS_ID = :key LIMIT :limit OFFSET :offset",
                param_array)

        images = []

        for img in results:
            images.append(img['ID'])

        return images

    def get_class_images(self, user, class_name, show, offset, db: Database = None):
        if db is None:
            db = Database()
        param_array = {'limit': show, 'offset': offset}
        if class_name is None or class_name == "-1" or class_name == -1:
            results = db.select_all(
                "SELECT * FROM "+self.learning_data_table+" WHERE CLASS_ID IS NULL OR CLASS_ID = -1 LIMIT :limit OFFSET :offset",
                param_array)
        else:
            param_array['key'] = class_name
            results = db.select_all(
                "SELECT * FROM "+self.learning_data_table+" WHERE CLASS_ID = :key LIMIT :limit OFFSET :offset",
                param_array)
        images = []
        for img in results:
            images.append(img['ID'])
        return images

    def upload_images(self, user, data, class_name, db: Database = None):
        if db is None:
            db = Database()
        images = json.loads(data)
        counter = 0
        if (class_name == "-1" or class_name == -1):
            img_class = None
        for image in images:
            filename = self.classifier_tag+"-" + str(int(time.time())) + "_" + str(counter)
            counter += 1
            image_path = FileUtils.create_image(filename, "people", image, optimize=True)
            db.insert("INSERT INTO PEOPLE_LEARNING_DATA (PATH, CLASS_ID) VALUES (:path, :class_id)",
                      {'path': image_path, 'class_id': class_name})

    def classify_image(self, user, image_data, db: Database = None):
        if db is None:
            db = Database()
        filename = "person-" + str(int(time.time()))
        image_path = FileUtils.create_image(filename, "people", image_data, optimize=True)
        db.insert("INSERT INTO PEOPLE_LEARNING_DATA (PATH) VALUES (:path)",
                  {'path': image_path})
        predictions = self.classifier.predict(os.path.join(Constants.DATA_DIR, image_path))
        for prediction in predictions:
            person_data = self.get_class_data(prediction['prediction'])
            Logger.log(person_data['NAME'] + ": " + str(prediction['confidence']))
        Logger.log(image_path)
        person_data = self.get_class_data(predictions[0]['prediction'])
        return {'label': person_data['NAME'], 'confidence': str(predictions[0]['confidence'] * 100)}


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.ControlManager.ThermostatManager import ThermostatManager

ACTION_KEY_HEATING_CONTROL = "heatingcontrol"
ACTION_KEY_CONTROL_ROOM_HEATING = "controlroomheating"
ACTION_KEY_GET_THERMOSTATS = "getthermostats"

class ThermostatAPIModule(APIModule):
    def __init__(self):
        super(ThermostatAPIModule, self).__init__()
        self.thermostat_manager = ThermostatManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_HEATING_CONTROL: self.heating_control,
            ACTION_KEY_CONTROL_ROOM_HEATING: self.control_room_heating,
            ACTION_KEY_GET_THERMOSTATS: self.get_thermostats
        }

        return mappings

    def heating_control(self, user, request, db):
        self.thermostat_manager.heating_control(user, request['type'], request['id'], request['value'], db)
        return Status(type=STATUS_OK)

    def control_room_heating(self, user, request, db):
        self.thermostat_manager.control_room_heating(user, request['location'], request['value'], db)
        return Status(type=STATUS_OK)

    def get_thermostats(self, user, request, db):
        data = self.thermostat_manager.get_thermostats(user, request['room'], db)
        return Status(type=STATUS_OK, data={'thermostats': data})


#!/usr/bin/python
# -*- coding: utf-8 -*-

from Homevee.Utils.Database import Database

class ToggleManager:
    def __init__(self):
        return

    def get_toggle_devices(self, user, room, db: Database = None):
        if db is None:
            db = Database()
        devices = []
        results = db.select_all("SELECT * FROM URL_TOGGLE WHERE LOCATION = :location",
                    {'location': room})

        for toggle in results:
            devices.append({'name': toggle['NAME'], 'id': toggle['ID'], 'type': 'URL-Toggle',
                'icon': toggle['ICON']})

        return devices


#!/usr/bin/python
# -*- coding: utf-8 -*-
from math import radians, cos, sqrt, atan2, sin

from Homevee.Utils.Database import Database

MIN_RETENTION_PERIOD = 5 * 60 * 1000 #5 minuten
MAX_DISTANCE = 500 #500 Meter

class GPSDataManager:
    def __init__(self):
        return

    def update_gps(self, user, time, lat, lng, db: Database = None):
        if db is None:
            db = Database()

        self.on_location_change(user, int(time), float(lat), float(lng))

        new_id = db.insert("INSERT INTO GPS_DATA (user, TIMESTAMP, LATITUDE, LONGITUDE) VALUES (:user, :time, :lat, :lng);",
                    {'user': user.username, 'time': time, 'lat': lat, 'lng': lng})

        if new_id != False:
            return True
        else:
            return False

    def on_location_change(self, user, time, lat, lng, db: Database = None):
        if db is None:
            db = Database()
        #determine user location (at work, at school, at home, etc.)
        current_user_location = self.determine_user_location(user, time, lat, lng)
        self.set_user_location(user, current_user_location)

        return

    def determine_user_location(self, user, time, lat, lng, db: Database = None):
        if db is None:
            db = Database()
        #if user is longer than MIN_RETENTION_PERIOD in roughly the same place => update his location status
        #check distance to place => less than MAX_DISTANCE => user is there

        place = self.get_nearest_place(user, lat, lng)

        location_data = db.select_all("SELECT * FROM GPS_DATA WHERE USERNAME = :user ORDER BY TIMESTAMP DESC LIMIT 50", {'user': user.username})
        if place is not None and place['DISTANCE'] <= MAX_DISTANCE:
            for location in location_data:
                if(self.compute_distance(place['LATITUDE'], place['LONGITUDE'], location['LATITUDE'], location['LONGITUDE']) <= MAX_DISTANCE):
                    if(time - int(location['TIMESTAMP']) >= MIN_RETENTION_PERIOD):
                        return place
                else:
                    break

        return None

    def get_nearest_place(self, user, lat, lng, db: Database = None):
        if db is None:
            db = Database()

        place = db.select_one("SELECT *, ((LATITUDE - :lat)*(LATITUDE - :lat)) + ((LONGITUDE - :lng)*(LONGITUDE - :lng)) as DISTANCE FROM PLACES ORDER BY DISTANCE ASC LIMIT 1",
                    {'lat': lat, 'lng': lng})

        return place

    def compute_distance(self, lat1, lng1, lat2, lng2):
        R = 6373000.0

        lat1 = radians(float(lat1))
        lng1 = radians(float(lng1))
        lat2 = radians(float(lat2))
        lng2 = radians(float(lng2))

        dlon = lng2 - lng1
        dlat = lat2 - lat1

        a = sin(dlat / 2) ** 2 + cos(lat1) * cos(lat2) * sin(dlon / 2) ** 2
        c = 2 * atan2(sqrt(a), sqrt(1 - a))

        distance = R * c

        return distance

    def set_user_location(self, user, place, db: Database = None):
        if db is None:
            db = Database()

        place_id = -1

        if(place is not None):
            place_id = place['ID']

        db.update("UPDATE USERDATA SET CURRENT_LOCATION = :loc WHERE USERNAME = :user",
                    {'loc': place_id, 'user': user.username})

    def get_gps_locations(self, user, db: Database = None):
        if db is None:
            db = Database()

        locations = []

        results = db.select_all("SELECT * FROM GPS_DATA WHERE USERNAME = :user", {'user': user.username})

        for item in results:
            if not self.is_near(locations, item['LATITUDE'], item['LONGITUDE']):
                locations.append({'time': item['TIMESTAMP'], 'latitude': item['LATITUDE'], 'longitude': item['LONGITUDE']})

        return locations

    def is_near(self, locations, lat, lng):
        MIN_DISTANCE_TO_OTHERS = 1000

        for location in locations:
            if (self.compute_distance(location['latitude'], location['longitude'], lat, lng) < MIN_DISTANCE_TO_OTHERS):
                return True
        return False


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.DashboardManager import DashboardManager

ACTION_KEY_GET_USER_DASHBOARD_ITEMS = "getuserdashboarditems"
ACTION_KEY_GET_USER_FAVOURITE_DEVICES = "getuserfavouritedevices"
ACTION_KEY_SET_EDIT_USER_DASHBOARD = "edituserdashboard"
ACTION_KEY_SET_GET_USER_DASHBOARD = "getuserdashboard"

class DashboardAPIModule(APIModule):
    def __init__(self):
        super(DashboardAPIModule, self).__init__()
        self.dashboard_manager = DashboardManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_USER_DASHBOARD_ITEMS: self.get_user_dashboard_items,
            ACTION_KEY_GET_USER_FAVOURITE_DEVICES: self.get_user_favourite_devices,
            ACTION_KEY_SET_EDIT_USER_DASHBOARD: self.edit_user_dashboard,
            ACTION_KEY_SET_GET_USER_DASHBOARD: self.get_user_dashboard
        }

        return mappings

    def get_user_dashboard_items(self, user, request, db):
        try:
            data = self.dashboard_manager.get_user_dashboard_items(user, db)
            if data is not None:
                return Status(type=STATUS_OK, data={'dashboard':data})
            else:
                return Status(type=STATUS_ERROR)
        except:
            return Status(type=STATUS_ERROR)

    def get_user_favourite_devices(self, user, request, db):
        try:
            data = self.dashboard_manager.get_user_favourite_devices(user, db)
            if data is not None:
                return Status(type=STATUS_OK, data={'dashboard':data})
            else:
                return Status(type=STATUS_ERROR)
        except:
            return Status(type=STATUS_ERROR)

    def edit_user_dashboard(self, user, request, db):
        try:
            success = self.dashboard_manager.edit_user_dashboard(user, request['dashboarddata'], db)
            if success:
                return Status(type=STATUS_OK)
            else:
                return Status(type=STATUS_ERROR)
        except:
            return Status(type=STATUS_ERROR)

    def get_user_dashboard(self, user, request, db):
        try:
            data = self.dashboard_manager.get_user_dashboard(user, db)
            if data is not None:
                return Status(type=STATUS_OK, data=data)
            else:
                return Status(type=STATUS_ERROR)
        except:
            return Status(type=STATUS_ERROR)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

from Homevee.Helper.helper_functions import get_my_ip
from Homevee.Item.Status import *
from Homevee.Utils.Database import Database


class MQTTConnectorManager:
    def __init__(self):
        return

    def generate_key(self):
        return "123456"

    def save_mqtt_device(self, user, type, location, id, data, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission(location):
            return {'result': 'nopermission'}

        #if type == MQTT_SENSOR:
        #    db.insert("INSERT INTO MQTT_SENSORS")

        item_data = json.loads(data)

        for item in item_data:
            if item['devicetype'] == "sensor":
                db.insert("INSERT INTO MQTT_SENSORS (NAME, ICON, TYPE, ROOM, SAVE_DATA, DEVICE_ID, VALUE_ID, LAST_VALUE) VALUES (:name, :icon, :type, :room, :save_data, :dev_id, :val_id, \"N/A\")",
                {'name': item['name'], 'icon': item['icon'], 'type': item['sensor_type'],
                'room': location, 'save_data': item['save_data'], 'dev_id': id, 'val_id': item['id']})

        return Status(type=STATUS_OK).get_dict()

    def generate_device_data(self, user, location, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission(location):
            return {'result': 'nopermission'}

        item = db.select_one("SELECT * FROM MQTT_DEVICES ORDER BY ID DESC", {})

        if item is not None:
            new_id = item['ID']+1
        else:
            new_id = 0

        topic = "/home/device/"+str(new_id)

        key = self.generate_key()

        db.insert("INSERT INTO MQTT_DEVICES (ID, LOCATION, KEY, TOPIC) VALUES (:id, :location, :key, :topic)",
                    {'id': new_id, 'location': location, 'key': key, 'topic': topic})

        return {'id': new_id, 'topic': topic, 'key': key, 'ip': get_my_ip(),
                'remoteid': db.get_server_data("REMOTE_ID")}

    def add_to_intermediates(self, id, db: Database = None):
        if db is None:
            db = Database()
            db.insert("INSERT INTO MQTT_DEVICE_INTERMEDIATES (ID) VALUES (:id)", {'id': id})

    def is_in_intermediates(self, id, db: Database = None):
        if db is None:
            db = Database()
            data = db.select_one("SELECT COUNT(*) FROM MQTT_DEVICE_INTERMEDIATES WHERE ID = :id", {'id': id})

            if data['COUNT(*)'] == 0:
                return False
            else:
                return True



#!/usr/bin/python
# -*- coding: utf-8 -*-

import datetime
import json

from Homevee.Utils.Database import Database


class WeatherManager:
    def __init__(self):
        return

    def get_weather(self, daycount, db: Database = None):
        if db is None:
            db = Database()

        try:
            if daycount > 16:
                daycount = 16

            weather_data = json.loads(db.get_server_data("WEATHER_CACHE"))

            days = weather_data['list']

            output = []

            for i in range(0, min(len(days), daycount)):
                day = days[i]

                relative_days = ["Heute", "Morgen", "Übermorgen"]
                day_names = ["Mo", "Di", "Mi", "Do", "Fr", "Sa", "So"]

                if i > 2:
                    date = datetime.datetime.fromtimestamp(day['dt'])

                    week_day_number = date.weekday()

                    date = day_names[week_day_number]+", "+date.strftime('%d.%m.')
                else:
                    date = relative_days[i]

                city_name = weather_data['city']['name']

                icon_link = "http://openweathermap.org/img/w/"+day['weather'][0]['icon']+".png"

                weather_desc = day['weather'][0]['description']
                icon = day['weather'][0]['icon']

                pressure = None
                if "pressure" in day:
                    pressure = day['pressure']

                humidity = None
                if "humidity" in day:
                    humidity = day['humidity']

                wind_speed = None
                if "speed" in day:
                    wind_speed = float(day['speed'])*3.6

                wind_direction = None
                if "deg" in day:
                    wind_direction = day['deg']

                clouds = None
                if "clouds" in day:
                    clouds = day['clouds']

                rain = None
                if "rain" in day:
                    rain = day['rain']

                snow = None
                if "snow" in day:
                    snow = day['snow']

                day_item = {'city':city_name, 'date':date, 'temps':day['temp'], 'icon':icon_link, 'iconid':icon,
                            'desc':weather_desc, 'pressure':pressure, 'humidity':humidity, 'windspeed':wind_speed,
                            'winddirection': wind_direction, 'clouds':clouds, 'rain':rain, 'snow':snow}

                output.append(day_item)

            return output
        except:
            return None

    def set_weather_city_id(self, request, db: Database = None):
        if db is None:
            db = Database()
        return None

    def get_weather_city_list(self, db):
        try:
            return db.get_server_data("WEATHER_CITY_CACHE")
        except:
            return None


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.MQTTConnectorManager import MQTTConnectorManager

ACTION_KEY_GENERATE_DEVICE_DATA = "generatedevicedata"
ACTION_KEY_SAVE_MQTT_DEVICE = "savemqttdevice"

class MQTTConnectorAPIModule(APIModule):
    def __init__(self):
        super(MQTTConnectorAPIModule, self).__init__()
        self.manager = MQTTConnectorManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GENERATE_DEVICE_DATA: self.generate_device_data,
            ACTION_KEY_SAVE_MQTT_DEVICE: self.save_mqtt_device
        }

        return mappings

    def generate_device_data(self, user, request, db):
        data = self.manager.generate_device_data(user, request['location'], db)
        return Status(type=STATUS_OK, data=data)

    def save_mqtt_device(self, user, request, db):
        data = self.manager.save_mqtt_device(user, request['type'], request['location'], request['id'],
                                                       request['devicedata'], db)
        return Status(type=STATUS_OK, data=data)


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.SystemInfoManager import SystemInfoManager

ACTION_KEY_GET_SYSTEM_INFO = "getsysteminfo"

class SystemInfoAPIModule(APIModule):
    def __init__(self):
        super(SystemInfoAPIModule, self).__init__()
        self.system_info_manager = SystemInfoManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_SYSTEM_INFO: self.get_system_info
        }
        return mappings

    def get_system_info(self, user, request, db) -> Status:
        try:
            system_info = self.system_info_manager.get_system_info()
            return Status(type=STATUS_OK, data={'systeminfo': system_info})
        except:
            return Status(type=STATUS_ERROR)



from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.GatewayManager import GatewayManager

ACTION_KEY_GET_GATEWAYS = "getgateways"
ACTION_KEY_ADD_EDIT_GATEWAY = "addeditgateway"
ACTION_KEY_DELETE_GATEWAY = "deletegateway"
ACTION_KEY_GET_GATEWAY_DEVICES = "getgatewaydevices"
ACTION_KEY_CONNECT_GATEWAY = "connectgateway"

class GatewayAPIModule(APIModule):
    def __init__(self):
        super(GatewayAPIModule, self).__init__()
        self.gateway_manager = GatewayManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_GATEWAYS: self.get_gateways,
            ACTION_KEY_ADD_EDIT_GATEWAY: self.add_edit_gateway,
            ACTION_KEY_DELETE_GATEWAY: self.delete_gateway,
            ACTION_KEY_GET_GATEWAY_DEVICES: self.get_gateway_devices,
            ACTION_KEY_CONNECT_GATEWAY: self.connect_gateway
        }

        return mappings

    def get_gateways(self, user, request, db):
        data = self.gateway_manager.get_gateways(user, db)
        return Status(type=STATUS_OK, data=data)

    def add_edit_gateway(self, user, request, db):
        data = self.gateway_manager.add_edit_gateway(user, request['type'], request['usr'], request['psw'],
                                                         request['changepw'],
                                                         request['ip'], request['port'], request['gateway_type'], db)
        return Status(type=STATUS_OK, data=data)

    def delete_gateway(self, user, request, db):
        data = self.gateway_manager.delete_gateway(user, request['type'], db)
        return Status(type=STATUS_OK, data=data)

    def get_gateway_devices(self, user, request, db):
        devices = self.gateway_manager.get_gateway_devices(user, request['type'], db)
        return Status(type=STATUS_OK, data={'devices': devices})

    def connect_gateway(self, user, request, db):
        data = self.gateway_manager.connect_gateway(user, request['type'], request['ip'], db)
        return Status(type=STATUS_OK, data=data)


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.DeviceAPI.get_modes import get_modes
from Homevee.Exception import NoPermissionException
from Homevee.Helper import Logger
from Homevee.Item import Item
from Homevee.Item.Room import Room
from Homevee.Item.User import User
from Homevee.Manager.ControlManager.BlindsManager import BlindsManager
from Homevee.Manager.ControlManager.DimmerManager import DimmerManager
from Homevee.Manager.ControlManager.EnergyDataManager import EnergyDataManager
from Homevee.Manager.ControlManager.RGBLightManager import RGBLightManager
from Homevee.Manager.ControlManager.ThermostatManager import ThermostatManager
from Homevee.Manager.ControlManager.ToggleManager import ToggleManager
from Homevee.Manager.ControlManager.WakeOnLanManager import WakeOnLanManager
from Homevee.Manager.SceneManager import SceneManager
from Homevee.Manager.SensorDataManager import SensorDataManager
from Homevee.Manager.TriggerManager import TriggerManager
from Homevee.Utils.Database import Database
from Homevee.Utils.DeviceTypes import *

CONTROL_TYPE_SWITCH = "switch"
CONTROL_TYPE_VALUE = "value"
CONTROL_TYPE_DIMMER = "dimmer"
CONTROL_TYPE_TOOGLE = "toggle"
CONTROL_TYPE_TRIGGER = "trigger"
CONTROL_TYPE_RGB = "rgb"
CONTROL_TYPE_HEATING = "heating"
CONTROL_TYPE_SCENES = "scenes"
CONTROL_TYPE_WOL = "wwakeonlan"
CONTROL_TYPE_XBOXONE = "xboxone"
CONTROL_TYPE_BLINDS = "blinds"



class RoomDataManager():
    def __init__(self):
        return

    def get_room_data(self, user: User, room, db: Database = None) -> list:
        if db is None:
            db = Database()
        if not user.has_permission(room):
            raise NoPermissionException

        roomdata = []

        room = Item.load_from_db(Room, room, db)

        #Schalter abfragen
        switchdata = get_modes(user, room, "", "")
        for item in switchdata:
            roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype':item['device_type'],
                'icon': item['icon'], 'type': CONTROL_TYPE_SWITCH, 'value': item['mode']})

        #Dimmer
        dimmerdata = DimmerManager().get_dimmers(user, room)
        for item in dimmerdata:
            roomdata.append({'name': item['name'], 'id': item['device'], 'devicetype': item['type'],
                             'icon': item['icon'], 'type': CONTROL_TYPE_DIMMER, 'value': item['value']})

        #Toggle
        toggledata = ToggleManager().get_toggle_devices(user, room)
        for item in toggledata:
            roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype':item['type'],
                'icon': item['icon'], 'type': CONTROL_TYPE_TOOGLE, 'value': ''})

        #Trigger
        triggerdata = TriggerManager().get_triggers(user, room)
        for item in triggerdata:
            roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype':item['type'],
                'icon': item['icon'], 'type': CONTROL_TYPE_TRIGGER, 'value': ''})

        #RGB
        rgbdata = RGBLightManager().get_rgb_devices(user, room)
        for item in rgbdata:
            Logger.log(item)
            roomdata.append({'name': item['name'], 'devicetype':item['type'], 'id': item['id'],
                'icon': item['icon'], 'type': CONTROL_TYPE_RGB, 'value': item['value']})

        #Thermostate
        thermostat_data = ThermostatManager().get_thermostats(user, room)
        for item in thermostat_data[0]['thermostat_array']:
            roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype': item['type'],
                             'icon': item['icon'], 'type': CONTROL_TYPE_HEATING, 'value': item['data']['value'],
                             'step': 0.5, 'min': item['data']['min'], 'max': item['data']['max']})

        #Sensoren abfragen
        sensordata = SensorDataManager().get_sensor_data(user, room)
        for item in sensordata[0]['value_array']:
            roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype': item['device_type'],
                             'icon': item['icon'], 'type': CONTROL_TYPE_VALUE, 'value': item['value']})

        #Szenen abfragen
        scenes = SceneManager().get_scenes(user, room)
        if len(scenes) > 0:
            roomdata.append({'name': "Szenen", 'id': "scenes", 'devicetype': "scenes",
                             'icon': "scenes", 'type': CONTROL_TYPE_SCENES, 'value': ""})

        #WOL abfragen
        wol_devices = WakeOnLanManager().get_wol_devices(user, room)
        for item in wol_devices:
            roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype': WAKE_ON_LAN,
                             'icon': item['icon'], 'type': CONTROL_TYPE_WOL})

        #XBOX ONE WOL abfragen
        xbox_devices = WakeOnLanManager().get_xbox_devices(user, room)
        for item in xbox_devices:
            roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype': XBOX_ONE_WOL,
                             'icon': item['icon'], 'type': CONTROL_TYPE_XBOXONE})

        #Jalousie-Steuerung
        blind_controls = BlindsManager().get_blinds(user, room)
        for item in blind_controls:
            roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype': item['type'],
                             'icon': item['icon'], 'type': CONTROL_TYPE_BLINDS, 'value': item['value']})

        #Strom-Messgeräte
        power_usage_devices = EnergyDataManager().get_power_usage_devices(user, room)
        for item in power_usage_devices:
            roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype': item['devicetype'],
                             'icon': item['icon'], 'type': CONTROL_TYPE_VALUE, 'value': item['value']})

        return roomdata


#!/usr/bin/python
# -*- coding: utf-8 -*-
import urllib.error
import urllib.parse
import urllib.request

from Homevee.DeviceAPI import philips_hue
from Homevee.Item.Status import *
from Homevee.Utils.Database import Database
from Homevee.Utils.DeviceTypes import *


class RGBLightManager:
    def __init__(self):
        return

    def rgb_control(self, user, devicetype, id, mode, brightness, color, db, check_user=True):
        if devicetype == URL_RGB_LIGHT:
            result = db.select_one("SELECT * FROM URL_RGB_LIGHT WHERE ID = :id", {'id': id})

            if check_user and not user.has_permission(result['LOCATION']):
                return {'result': 'nopermission'}

            output = urllib.request.urlopen("http://"+result['URL']+color).read()

            return Status(type=STATUS_OK).get_dict()
        elif devicetype == PHILIPS_HUE_LIGHT:
            return philips_hue.set_light_mode(user, id, mode, None, brightness, color, db, check_user=True)

    def get_rgb_devices(self, user, room, db: Database = None):
        if db is None:
            db = Database()
        devices = []

        #URL_RGB_LIGHT
        results = db.select_all("SELECT * FROM URL_RGB_LIGHT WHERE LOCATION = :room", {'room': room})
        for item in results:
            rgb_item = {'name': item['NAME'], 'id': item['ID'], 'type': URL_RGB_LIGHT,
            'icon': item['ICON'], 'value': {'is_on': False, 'brightness': 100, 'color': item['LAST_COLOR']}}
            devices.append(rgb_item)
        #PHILIPS_HUE
        results = db.select_all("SELECT * FROM PHILIPS_HUE_LIGHTS WHERE LOCATION = :room AND TYPE = 'color'", {'room': room})
        for item in results:
            rgb_item = {'name': item['NAME'], 'id': item['ID'], 'type': PHILIPS_HUE_LIGHT,
                        'icon': item['ICON'], 'value': {'is_on': item['IS_ON']== 1, 'brightness': int(item['BRIGHTNESS']/255*100), 'color': item['HUE']}}
            devices.append(rgb_item)

        return devices

    def get_rgb_device(self, user, type, id, db: Database = None):
        if db is None:
            db = Database()

            #URL_RGB_LIGHT
            if type == URL_RGB_LIGHT:
                item = db.select_one("SELECT * FROM URL_RGB_LIGHT WHERE ID = :id", {'id': id})
                rgb_item = {'name': item['NAME'], 'id': item['ID'], 'type': URL_RGB_LIGHT,
                'icon': item['ICON'], 'value': {'is_on': False, 'brightness': 100, 'color': item['LAST_COLOR']}}
                return rgb_item
            #PHILIPS_HUE
            elif type == PHILIPS_HUE_LIGHT:
                item = db.select_one("SELECT * FROM PHILIPS_HUE_LIGHTS WHERE ID = :id AND TYPE = 'color'", {'id': id})

                rgb_item = {'name': item['NAME'], 'id': item['ID'], 'type': PHILIPS_HUE_LIGHT,
                            'icon': item['ICON'], 'value': {'is_on': item['IS_ON']== 1,
                            'brightness': int(item['BRIGHTNESS']/255*100), 'color': item['HUE']}}
                return rgb_item


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

from Homevee.Item.Status import *
from Homevee.Utils.Database import Database


class VoiceReplaceManager:
    def __init__(self):
        return

    def get_voice_replace_items(self, user, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission("admin"):
            return Status(type=STATUS_NO_ADMIN).get_dict()

        replace_data = []

        results = db.select_all("SELECT DISTINCT REPLACE_WITH FROM VOICE_COMMAND_REPLACE WHERE USERNAME = :user",
                    {'user': user.username})

        for item in results:
            items = db.select_all("SELECT TEXT FROM VOICE_COMMAND_REPLACE WHERE REPLACE_WITH == :item AND USERNAME = :user",
                {'user': user.username, 'item': item['REPLACE_WITH']})

            replacements = []

            for replacement in items:
                replacements.append(replacement['TEXT'])

            replace_data.append({'replacewith': item['REPLACE_WITH'], 'replacearray': replacements})

        return replace_data

    def add_edit_voice_replace_item(self, user, replacewith, replaceitems, db: Database = None) -> bool:
        if db is None:
            db = Database()
        replaceitems = json.loads(replaceitems)

        try:
            db.delete("DELETE FROM VOICE_COMMAND_REPLACE WHERE REPLACE_WITH = :replacewith AND USERNAME = :user",
                        {'replacewith': replacewith, 'user': user.username})

            for item in replaceitems:
                db.insert("INSERT INTO VOICE_COMMAND_REPLACE (USERNAME, REPLACE_WITH, TEXT) VALUES (:user, :replacewith, :text)",
                            {'user': user.username, 'replacewith': replacewith, 'text': item})
            return True
        except:
            return False

    def delete_voice_replace_item(self, user, replacewith, db: Database = None) -> bool:
        if db is None:
            db = Database()
        return db.delete("DELETE FROM VOICE_COMMAND_REPLACE WHERE REPLACE_WITH = :replacewith AND USERNAME = :user",
                    {'replacewith': replacewith, 'user': user.username})


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.CustomVoiceCommandManager import CustomVoiceCommandManager

ACTION_KEY_GET_VOICE_COMMANDS = "getvoicecommands"
ACTION_KEY_ADD_EDIT_VOICE_COMMAND = "addeditvoicecommand"
ACTION_KEY_DELETE_VOICE_COMMAND = "deletevoicecommand"

class CustomVoiceCommandAPIModule(APIModule):
    def __init__(self):
        super(CustomVoiceCommandAPIModule, self).__init__()
        self.voice_command_manager = CustomVoiceCommandManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_VOICE_COMMANDS: self.get_voice_commands,
            ACTION_KEY_ADD_EDIT_VOICE_COMMAND: self.add_edit_voice_command,
            ACTION_KEY_DELETE_VOICE_COMMAND: self.delete_voice_command
        }

        return mappings

    def get_voice_commands(self, user, request, db):
        data = self.voice_command_manager.get_voice_commands(db)
        return Status(type=STATUS_OK, data={'commands': data})

    def add_edit_voice_command(self, user, request, db):
        self.voice_command_manager.add_edit_voice_command(user, request['id'], request['name'],
                                                                          request['command_data'],
                                                                          request['response_data'],
                                                                          request['action_data'], db)
        return Status(type=STATUS_OK)

    def delete_voice_command(self, user, request, db):
        self.voice_command_manager.delete_voice_command(user, request['id'], db)
        return Status(type=STATUS_OK)


from Homevee.APIModule import APIModule
from Homevee.Item.Status import *
from Homevee.Manager.GraphDataManager import GraphDataManager

ACTION_KEY_GET_GRAPH_DATA = "getgraphdata"

class GraphDataAPIModule(APIModule):
    def __init__(self):
        super(GraphDataAPIModule, self).__init__()
        self.graph_data_manager = GraphDataManager()
        return

    def get_function_mappings(self):
        mappings = {
            ACTION_KEY_GET_GRAPH_DATA: self.get_graph_data,
        }
        return mappings

    def get_graph_data(self, user, request, db) -> Status:
        graph_data = self.graph_data_manager.get_graph_data(user, request['room'], request['type'],
                                                         request['id'], request['von'], request['bis'], db)
        return Status(type=STATUS_OK, data=graph_data)